@@ -2,19 +2,39 @@
 
 """
 #============================================================
-__version__ = "$Revision: 1.24 $"
+__version__ = "$Revision: 1.27 $"
 __author__ = "Carlos Moro <cfmoro1976@yahoo.es>, Karsten Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = 'GPL (for details see http://gnu.org)'
 
 import sys
 
+
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-
 from Gnumed.pycommon import gmLog, gmPG2, gmExceptions, gmBusinessDBObject
 
+
 _log = gmLog.gmDefLog
 _log.Log(gmLog.lInfo, __version__)
+
+_ = lambda x:x
+
+soap_cat2l10n = {
+	's': _('soap_S').strip(u'soap_'),
+	'o': _('soap_O').strip(u'soap_'),
+	'a': _('soap_A').strip(u'soap_'),
+	'p': _('soap_P').strip(u'soap_'),
+	None: _('soap_ADMIN').strip(u'soap_')
+}
+
+l10n2soap_cat = {
+	_('soap_S').strip(u'soap_'): 's',
+	_('soap_O').strip(u'soap_'): 'o',
+	_('soap_A').strip(u'soap_'): 'a',
+	_('soap_P').strip(u'soap_'): 'p',
+	_('soap_ADMIN').strip(u'soap_'): None
+}
+
 #============================================================
 class cDiag(gmBusinessDBObject.cBusinessDBObject):
 	"""Represents one real diagnosis.
@@ -167,6 +187,10 @@ if __name__ == '__main__':
 	_log = gmLog.gmDefLog
 	_log.SetAllLogLevels(gmLog.lData)
 
+	from Gnumed.pycommon import gmI18N
+	gmI18N.activate_locale()
+	gmI18N.install_domain(domain = 'gnumed')
+
 	def test_diag():
 		print "\nDiagnose test"
 		print  "-------------"
@@ -202,6 +226,15 @@ if __name__ == '__main__':
 	
 #============================================================
 # $Log: gmClinNarrative.py,v $
+# Revision 1.27  2007/11/05 12:09:29  ncq
+# - support admin soap type
+#
+# Revision 1.26  2007/09/10 12:31:55  ncq
+# - improve test suite
+#
+# Revision 1.25  2007/09/07 22:36:44  ncq
+# - soap_cat2l10n and back
+#
 # Revision 1.24  2006/11/20 15:55:12  ncq
 # - gmPG2.run_ro_queries *always* returns (rows, idx) so be aware of that
 #
