@@ -3,20 +3,20 @@
 license: GPL
 """
 #============================================================
-__version__ = "$Revision: 1.99 $"
+__version__ = "$Revision: 1.103 $"
 __author__ = "Carlos Moro <cfmoro1976@yahoo.es>"
 
 import types, sys, string, datetime
 
+
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-
 from Gnumed.pycommon import gmLog, gmPG2, gmExceptions, gmNull, gmBusinessDBObject, gmDateTime
 from Gnumed.business import gmClinNarrative
 
+
 _log = gmLog.gmDefLog
 _log.Log(gmLog.lInfo, __version__)
-
 #============================================================
 class cHealthIssue(gmBusinessDBObject.cBusinessDBObject):
 	"""Represents one health issue.
@@ -121,6 +121,20 @@ class cHealthIssue(gmBusinessDBObject.cBusinessDBObject):
 			return None
 		return cEpisode(aPK_obj=rows[0][0])
 #============================================================
+# use as dummy for unassociated episodes
+def get_dummy_health_issue():
+	issue = {
+		'pk': None,
+		'description': _('free-standing episodes'),
+		'age_noted': None,
+		'laterality': u'na',
+		'is_active': True,
+		'clinically_relevant': True,
+		'is_confidential': None,
+		'is_cause_of_death': False
+	}
+	return issue
+#============================================================
 class cEpisode(gmBusinessDBObject.cBusinessDBObject):
 	"""Represents one clinical episode.
 	"""
@@ -146,7 +160,10 @@ class cEpisode(gmBusinessDBObject.cBusinessDBObject):
 	def __init__(self, aPK_obj=None, id_patient=None, name='xxxDEFAULTxxx', health_issue=None, row=None):
 		pk = aPK_obj
 		if pk is None and row is None:
-			cmd = u"select * from clin.v_pat_episodes where pk_patient=%s and description=%s and pk_health_issue=%s"
+			if health_issue is None:
+				cmd = u"select * from clin.v_pat_episodes where pk_patient=%s and description=%s and pk_health_issue is %s"
+			else:
+				cmd = u"select * from clin.v_pat_episodes where pk_patient=%s and description=%s and pk_health_issue=%s"
 			rows, idx = gmPG2.run_ro_queries(queries = [{
 					'cmd': cmd,
 					'args': [id_patient, name, health_issue]
@@ -305,6 +322,40 @@ class cEncounter(gmBusinessDBObject.cBusinessDBObject):
 select exists (
 	select 1 from clin.v_pat_items where pk_patient=%(pat)s and pk_encounter=%(enc)s union all
 	select 1 from blobs.doc_med where fk_identity=%(pat)s and fk_encounter=%(enc)s
+)"""
+		args = {
+			'pat': self._payload[self._idx['pk_patient']],
+			'enc': self.pk_obj
+		}
+		rows, idx = gmPG2.run_ro_queries (
+			queries = [{
+				'cmd': cmd,
+				'args': args
+			}]
+		)
+		return rows[0][0]
+	#--------------------------------------------------------
+	def has_narrative(self):
+		cmd = u"""
+select exists (
+	select 1 from clin.v_pat_items where pk_patient=%(pat)s and pk_encounter=%(enc)s
+)"""
+		args = {
+			'pat': self._payload[self._idx['pk_patient']],
+			'enc': self.pk_obj
+		}
+		rows, idx = gmPG2.run_ro_queries (
+			queries = [{
+				'cmd': cmd,
+				'args': args
+			}]
+		)
+		return rows[0][0]
+	#--------------------------------------------------------
+	def has_documents(self):
+		cmd = u"""
+select exists (
+	select 1 from blobs.doc_med where fk_identity=%(pat)s and fk_encounter=%(enc)s
 )"""
 		args = {
 			'pat': self._payload[self._idx['pk_patient']],
@@ -394,20 +445,34 @@ def create_health_issue(patient_id=None, description=None):
 
 	return (True, h_issue)
 #-----------------------------------------------------------
-def create_episode(pk_health_issue=None, episode_name=None, patient_id=None, is_open=False):
+def delete_health_issue(health_issue=None):
+	if isinstance(health_issue, cHealthIssue):
+		pk = health_issue['pk']
+	else:
+		pk = int(health_issue)
+
+	try:
+		gmPG2.run_rw_queries(queries = [{'cmd': u'delete from clin.health_issue where pk=%(pk)s', 'args': {'pk': pk}}])
+	except gmPG2.dbapi.IntegrityError:
+		# should be parsing pgcode/and or error message
+		_log.LogException('cannot delete health issue')
+		raise gmExceptions.DatabaseObjectInUseError('cannot delete health issue, it is in use')
+#-----------------------------------------------------------
+def create_episode(pk_health_issue=None, episode_name=None, patient_id=None, is_open=False, allow_dupes=False):
 	"""Creates a new episode for a given patient's health issue.
 
 	pk_health_issue - given health issue PK
 	episode_name - name of episode
 	"""
-	try:
-		episode = cEpisode(id_patient=patient_id, name=episode_name, health_issue=pk_health_issue)
-		if episode['episode_open'] != is_open:
-			episode['episode_open'] = is_open
-			episode.save_payload()
-		return episode
-	except gmExceptions.ConstructorError:
-		pass
+	if not allow_dupes:
+		try:
+			episode = cEpisode(id_patient=patient_id, name=episode_name, health_issue=pk_health_issue)
+			if episode['episode_open'] != is_open:
+				episode['episode_open'] = is_open
+				episode.save_payload()
+			return episode
+		except gmExceptions.ConstructorError:
+			pass
 
 	queries = []
 	cmd = u"insert into clin.episode (fk_health_issue, fk_patient, description, is_open) values (%s, %s, %s, %s::boolean)"
@@ -565,6 +630,22 @@ if __name__ == '__main__':
 
 #============================================================
 # $Log: gmEMRStructItems.py,v $
+# Revision 1.103  2007/10/29 11:04:11  ncq
+# - properly handle NULL pk_health_issue in create_apisode() thereby
+#   finding dupes in free-standing episodes, too
+# - this then asks for an explicit allow_dupes (defaulted to False)
+#   in create_episode() as we may, indeed, wish to allow dupes
+#   sometimes
+#
+# Revision 1.102  2007/10/11 12:00:17  ncq
+# - add has_narrative() and has_documents()
+#
+# Revision 1.101  2007/09/07 10:55:55  ncq
+# - get_dummy_health_issue()
+#
+# Revision 1.100  2007/08/15 14:56:30  ncq
+# - delete_health_issue()
+#
 # Revision 1.99  2007/05/18 13:25:56  ncq
 # - fix cEncounter.transfer_clinical_data()
 #
