@@ -4,17 +4,18 @@
 """
 #============================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/business/gmMedDoc.py,v $
-# $Id: gmMedDoc.py,v 1.94 2007/05/20 01:27:31 ncq Exp $
-__version__ = "$Revision: 1.94 $"
+# $Id: gmMedDoc.py,v 1.100 2007/11/05 11:36:29 ncq Exp $
+__version__ = "$Revision: 1.100 $"
 __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
 
-import sys, tempfile, os, shutil, os.path, types, time
+import sys, os, shutil, os.path, types, time
 from cStringIO import StringIO
 
+
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
+from Gnumed.pycommon import gmLog, gmExceptions, gmBusinessDBObject, gmPG2, gmTools, gmMimeLib
 
-from Gnumed.pycommon import gmLog, gmExceptions, gmBusinessDBObject, gmPG2, gmTools
 
 _log = gmLog.gmDefLog
 _log.Log(gmLog.lInfo, __version__)
@@ -156,123 +157,47 @@ class cMedDocPart(gmBusinessDBObject.cBusinessDBObject):
 		'filename'
 	]
 	#--------------------------------------------------------
-	def __del__(self):
-		if self.__dict__.has_key('__conn'):
-			self.__conn.close()
+#	def __del__(self):
+#		if self.__dict__.has_key('__conn'):
+#			self.__conn.close()
 	#--------------------------------------------------------
 	# retrieve data
 	#--------------------------------------------------------
 	def export_to_file(self, aTempDir = None, aChunkSize = 0, filename=None):
-		"""Export binary object data into file.
 
-			- returns file name or None
-		"""
 		if self._payload[self._idx['size']] == 0:
 			return None
 
 		if filename is None:
-			# if tempdir is None -> use tempfile module default, else use given
-			# path as base directory for temp files
-			if aTempDir is not None:
-				tempfile.tempdir = aTempDir
-			tempfile.template = "gm-doc_obj-page_%s" % self._payload[self._idx['seq_idx']]
-			filename = tempfile.mktemp()
+			suffix = None
+			# preserve original filename extension if available
+			if self._payload[self._idx['filename']] is not None:
+				name, suffix = os.path.splitext(self._payload[self._idx['filename']])
+				suffix = suffix.strip()
+				if suffix == u'':
+					suffix = None
+			# get unique filename
+			filename = gmTools.get_unique_filename (
+				prefix = 'gm-doc_obj-page_%s-' % self._payload[self._idx['seq_idx']],
+				suffix = suffix,
+				dir = aTempDir
+			)
 
-		# binary, no encoding
-		aFile = open(filename, 'wb+')
+		success = gmPG2.bytea2file (
+			data_query = {
+				'cmd': u'SELECT substring(data from %(start)s for %(size)s) FROM blobs.doc_obj WHERE pk=%(pk)s',
+				'args': {'pk': self.pk_obj}
+			},
+			filename = filename,
+			chunk_size = aChunkSize,
+			data_size = self._payload[self._idx['size']]
+		)
 
-		self.__export(aFile, aChunkSize)
-		aFile.close()
-		return filename
-	#--------------------------------------------------------
-	def export_to_string (self, aChunkSize = 0):
-		"""Returns the document as a Python string.
+		if success:
+			return filename
 
-			- WARNING: better have enough RAM for whatever size it is !!
-		"""
-		if self._payload[self._idx['size']] == 0:
-			return None
-		aFile = StringIO()
-		if self.__export(aFile, aChunkSize):
-			r = aFile.getvalue()
-			aFile.close()
-			return r
-		aFile.close()
 		return None
 	#--------------------------------------------------------
-	def __export (self, aFile=None, aChunkSize = 0):
-		"""Export binary object data into <aFile>.
-
-			- internal helper
-			- writes data to the Python file-like object <aFile>
-		"""
-		# If the client sets an encoding other than the default we
-		# will receive encoding-parsed data which isn't the binary
-		# content we want. Hence we need to get our own connection.
-		# It must be a read-write one so that we don't affect the
-		# encoding for other users of the shared read-only
-		# connections.
-		# Actually, encodings shouldn't be applied to binary data
-		# (eg. bytea types) in the first place but that is only
-		# reported to be fixed > v7.4.
-		# further tests reveal that at least on PG 8.0 this bug still
-		# manifests itself
-		conn = gmPG2.get_raw_connection()
-		# this shouldn't actually, really be necessary
-#		if conn.version > '7.4':
-#			print "****************************************************"
-#			print "*** if exporting BLOBs suddenly fails and        ***"
-#			print "*** you are running PostgreSQL >= 8.1 please     ***"
-#			print "*** mail a bug report to Karsten.Hilbert@gmx.net ***"
-#			print "****************************************************"
-
-		# Windoze sucks: it can't transfer objects of arbitrary size,
-		# or maybe this is due to pyPgSQL ?
-		# anyways, we need to split the transfer,
-		# however, only possible if postgres >= 7.2
-		max_chunk_size = aChunkSize
-
-		# a chunk size of 0 means: all at once
-		if ((max_chunk_size == 0) or (self._payload[self._idx['size']] <= max_chunk_size)):
-			# retrieve binary field
-			cmd = u"SELECT data FROM blobs.doc_obj WHERE pk=%s"
-			rows, idx = gmPG2.run_ro_queries(link_obj=conn, queries=[{'cmd': cmd, 'args': [self.pk_obj]}])
-			conn.close()
-			aFile.write(str(rows[0][0]))
-			return True
-
-		# retrieve chunks
-		# does this not carry the danger of cutting up multi-byte escape sequences ?
-		# no, since bytea is binary
-		needed_chunks, remainder = divmod(self._payload[self._idx['size']], max_chunk_size)
-		_log.Log(gmLog.lData, "%s chunks of %s bytes, remainder of %s bytes" % (needed_chunks, max_chunk_size, remainder))
-		for chunk_id in range(needed_chunks):
-			pos = (chunk_id*max_chunk_size) + 1
-			cmd = u"SELECT substring(data from %s for %s) FROM blobs.doc_obj WHERE pk=%%s" % (pos, max_chunk_size)
-			try:
-				rows, idx = gmPG2.run_ro_queries(link_obj=conn, queries=[{'cmd': cmd, 'args': [self.pk_obj]}])
-			except:
-				_log.Log(gmLog.lErr, 'cannot retrieve chunk [%s/%s], size [%s], doc part [%s], try decreasing chunk size' % (chunk_id+1, needed_chunks, max_chunk_size, self.pk_obj))
-				raise
-			# it would be a fatal error to see more than one result as ids are supposed to be unique
-			aFile.write(str(rows[0][0]))
-
-		# retrieve remainder
-		if remainder > 0:
-			_log.Log(gmLog.lData, "retrieving trailing bytes after chunks")
-			pos = (needed_chunks*max_chunk_size) + 1
-			cmd = u"SELECT substring(data from %s for %s) FROM blobs.doc_obj WHERE pk=%%s" % (pos, remainder)
-			try:
-				rows, idx = gmPG2.run_ro_queries(link_obj=conn, queries=[{'cmd': cmd, 'args': [self.pk_obj]}])
-			except:
-				_log.Log(gmLog.lErr, 'cannot retrieve remaining [%s] bytes from doc part [%s]' % (remainder, self.pk_obj))
-				raise
-			# it would be a fatal error to see more than one result as ids are supposed to be unique
-			aFile.write(str(rows[0][0]))
-
-		conn.close()
-		return True
-	#--------------------------------------------------------
 	def get_reviews(self):
 		cmd = u"""
 select
@@ -304,20 +229,11 @@ order by
 			_log.Log(gmLog.lErr, '[%s] is not a readable file' % fname)
 			return False
 
-		# read from file and convert (escape)
-		aFile = file(fname, "rb")
-		byte_str_img_data = aFile.read()
-		aFile.close()
-		del aFile
-		img_obj = buffer(byte_str_img_data)
-		del(byte_str_img_data)
-
-		conn = gmPG2.get_raw_connection()
-
-		# insert the data
-		cmd = u"UPDATE blobs.doc_obj SET data=%s::bytea WHERE pk=%s"
-		gmPG2.run_rw_queries(link_obj=conn, queries = [{'cmd': cmd, 'args': [img_obj, self.pk_obj]}], end_tx=True)
-		conn.close()
+		gmPG2.file2bytea (
+			query = u"UPDATE blobs.doc_obj SET data=%(data)s::bytea WHERE pk=%(pk)s",
+			filename = fname,
+			args = {'pk': self.pk_obj}
+		)
 
 		# must update XMIN now ...
 		self.refetch_payload()
@@ -390,6 +306,18 @@ where
 		self._payload[self._idx['seq_idx']] = rows[0][0]
 		self._is_modified = True
 		self.save_payload()
+	#--------------------------------------------------------
+	def display_via_mime(self, tmpdir=None, chunksize=0, block=None):
+
+		fname = self.export_to_file(aTempDir = tmpdir, aChunkSize = chunksize)
+		if fname is None:
+			return False, ''
+
+		success, msg = gmMimeLib.call_viewer_on_file(fname, block = block)
+		if not success:
+			return False, msg
+
+		return True, ''
 #============================================================
 class cMedDoc(gmBusinessDBObject.cBusinessDBObject):
 	"""Represents one medical document."""
@@ -453,7 +381,7 @@ class cMedDoc(gmBusinessDBObject.cBusinessDBObject):
 	def add_part(self, file=None):
 		"""Add a part to the document."""
 		# create dummy part
-		cmd1 = u"""
+		cmd = u"""
 			insert into blobs.doc_obj (
 				fk_doc, fk_intended_reviewer, data, seq_idx
 			) VALUES (
@@ -464,7 +392,7 @@ class cMedDoc(gmBusinessDBObject.cBusinessDBObject):
 			)"""
 		rows, idx = gmPG2.run_rw_queries (
 			queries = [
-				{'cmd': cmd1, 'args': {'doc_id': self.pk_obj}},
+				{'cmd': cmd, 'args': {'doc_id': self.pk_obj}},
 				{'cmd': u"select currval('blobs.doc_obj_pk_seq')"}
 			],
 			return_data = True
@@ -620,6 +548,13 @@ def search_for_document(patient_id=None, type_id=None):
 		docs.append(cMedDoc(row[0]))
 	return docs
 #------------------------------------------------------------
+def delete_document(document_id=None, encounter_id=None):
+	# will cascade to doc_obj and doc_desc
+	cmd = u"select blobs.delete_document(%(pk)s, %(enc)s)"
+	args = {'pk': document_id, 'enc': encounter_id}
+	rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
+	return
+#------------------------------------------------------------
 def get_document_types():
 	rows, idx = gmPG2.run_ro_queries (
 		queries = [{'cmd': u"SELECT * FROM blobs.v_doc_type"}],
@@ -666,10 +601,10 @@ def delete_document_type(document_type=None):
 #------------------------------------------------------------
 def get_ext_ref():
 	"""This needs *considerably* more smarts."""
-	# set up temp file environment for creating unique random directory
-	tempfile.template = ''
-	# create temp dir name
-	dirname = tempfile.mktemp(suffix = time.strftime(".%Y%m%d-%H%M%S", time.localtime()))
+	dirname = gmTools.get_unique_filename (
+		prefix = '',
+		suffix = time.strftime(".%Y%m%d-%H%M%S", time.localtime())
+	)
 	# extract name for dir
 	path, doc_ID = os.path.split(dirname)
 	return doc_ID
@@ -745,6 +680,28 @@ if __name__ == '__main__':
 
 #============================================================
 # $Log: gmMedDoc.py,v $
+# Revision 1.100  2007/11/05 11:36:29  ncq
+# - use blobs.delete_document()
+#
+# Revision 1.99  2007/10/31 22:06:44  ncq
+# - delete_document()
+#
+# Revision 1.98  2007/10/12 14:15:55  ncq
+# - cleanup
+#
+# Revision 1.97  2007/08/11 23:53:19  ncq
+# - cleanup
+# - use gmPG2.bytea2file()
+#
+# Revision 1.96  2007/08/09 07:58:44  ncq
+# - make export_to_file() use gmPG2.bytea2file()
+# - no more export_to_string()
+# - comment out __export()
+# - add display_via_mime()
+#
+# Revision 1.95  2007/07/11 21:02:27  ncq
+# - use gmTools.get_unique_filename()
+#
 # Revision 1.94  2007/05/20 01:27:31  ncq
 # - fix set_translation() - lang needs to go first in i18n.upd_tx()
 #
