@@ -35,9 +35,9 @@ This is useful in fields such as medicine where only partial
 timestamps may be known for certain events.
 """
 #===========================================================================
-# $Id: gmDateTime.py,v 1.9 2007/05/21 17:13:12 ncq Exp $
+# $Id: gmDateTime.py,v 1.14 2007/09/04 23:28:06 ncq Exp $
 # $Source: /sources/gnumed/gnumed/gnumed/client/pycommon/gmDateTime.py,v $
-__version__ = "$Revision: 1.9 $"
+__version__ = "$Revision: 1.14 $"
 __author__ = "K. Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL (details at http://www.gnu.org)"
 
@@ -158,7 +158,7 @@ def init():
 	)
 
 #===========================================================================
-def __explicit_offset(str_timestamp, offset_chars='hdwmy'):
+def __explicit_offset(str2parse, offset_chars='hdwmy'):
 	"""
 			Default is 'hdwm':
 			h - hours
@@ -170,17 +170,17 @@ def __explicit_offset(str_timestamp, offset_chars='hdwmy'):
 		This also defines the significance of the order of the characters.
 	"""
 	# "+/-XXd/w/m/t"
-	if not regex.match("^(\s|\t)*(\+|-)?(\s|\t)*\d{1,2}(\s|\t)*[%s](\s|\t)*$" % offset_chars, str_timestamp):
+	if not regex.match(u"^(\s|\t)*(\+|-)?(\s|\t)*\d{1,2}(\s|\t)*[%s](\s|\t)*$" % offset_chars, str2parse, flags = regex.LOCALE | regex.UNICODE):
 		return []
-	val = int(regex.findall('\d{1,2}', str_timestamp)[0])
-	offset_char = regex.findall('[%s]' % offset_chars, str_timestamp)[0].lower()
+	val = int(regex.findall(u'\d{1,2}', str2parse, flags = regex.LOCALE | regex.UNICODE)[0])
+	offset_char = regex.findall(u'[%s]' % offset_chars, str2parse, flags = regex.LOCALE | regex.UNICODE)[0].lower()
 
 	now = mxDT.now()
 	enc = gmI18N.get_encoding()
 
 	# allow past ?
 	is_future = True
-	if str_timestamp.find('-') > -1:
+	if str2parse.find('-') > -1:
 		is_future = False
 
 	ts = None
@@ -239,7 +239,7 @@ def __explicit_offset(str_timestamp, offset_chars='hdwmy'):
 	}
 	return [tmp]
 #---------------------------------------------------------------------------
-def __single_char(str_timestamp, trigger_chars='ndmy'):
+def __single_char(str2parse, trigger_chars='ndmy'):
 	"""This matches on single characters.
 
 	Spaces and tabs are discarded.
@@ -252,9 +252,9 @@ def __single_char(str_timestamp, trigger_chars='ndmy'):
 
 	This also defines the significance of the order of the characters.
 	"""
-	if not regex.match('^(\s|\t)*[%s]{1}(\s|\t)*$' % trigger_chars, str_timestamp):
+	if not regex.match(u'^(\s|\t)*[%s]{1}(\s|\t)*$' % trigger_chars, str2parse, flags = regex.LOCALE | regex.UNICODE):
 		return []
-	val = str_timestamp.strip().lower()
+	val = str2parse.strip().lower()
 
 	now = mxDT.now()
 	enc = gmI18N.get_encoding()
@@ -306,7 +306,7 @@ def __single_char(str_timestamp, trigger_chars='ndmy'):
 
 	return []
 #---------------------------------------------------------------------------
-def __single_slash(str_timestamp):
+def __single_slash(str2parse):
 	"""Expand fragments containing a single slash.
 
 	"5/"
@@ -324,8 +324,8 @@ def __single_slash(str_timestamp):
 	"""
 	matches = []
 	now = mxDT.now()
-	if regex.match("^(\s|\t)*\d{1,2}(\s|\t)*/+(\s|\t)*$", str_timestamp):
-		val = int(regex.findall('', str_timestamp))
+	if regex.match(u"^(\s|\t)*\d{1,2}(\s|\t)*/+(\s|\t)*$", str2parse, flags = regex.LOCALE | regex.UNICODE):
+		val = int(regex.findall(u'\d+', str2parse, flags = regex.LOCALE | regex.UNICODE)[0])
 
 		if val < 100 and val >= 0:
 			matches.append ({
@@ -385,8 +385,8 @@ def __single_slash(str_timestamp):
 				'label': '%.2d/19' % val
 			})
 
-	elif regex.match("^(\s|\t)*\d{1,2}(\s|\t)*/+(\s|\t)*\d{4}(\s|\t)*$", str_timestamp):
-		parts = regex.findall('\d+', str_timestamp)
+	elif regex.match(u"^(\s|\t)*\d{1,2}(\s|\t)*/+(\s|\t)*\d{4}(\s|\t)*$", str2parse, flags = regex.LOCALE | regex.UNICODE):
+		parts = regex.findall(u'\d+', str2parse, flags = regex.LOCALE | regex.UNICODE)
 		fts = cFuzzyTimestamp (
 			timestamp = mxDT.now() + mxDT.RelativeDateTime(year = int(parts[1]), month = int(parts[0])),
 			accuracy = acc_months
@@ -398,19 +398,19 @@ def __single_slash(str_timestamp):
 
 	return matches
 #---------------------------------------------------------------------------
-def __numbers_only(str_timestamp):
+def __numbers_only(str2parse):
 	"""This matches on single numbers.
 
 	Spaces or tabs are discarded.
 	"""
-	if not regex.match("^(\s|\t)*\d{1,4}(\s|\t)*$", str_timestamp):
+	if not regex.match(u"^(\s|\t)*\d{1,4}(\s|\t)*$", str2parse, flags = regex.LOCALE | regex.UNICODE):
 		return []
 
 	# strftime() returns str but in the localized encoding,
 	# so we may need to decode that to unicode
 	enc = gmI18N.get_encoding()
 	now = mxDT.now()
-	val = int(regex.findall('\d{1,4}', str_timestamp)[0])
+	val = int(regex.findall(u'\d{1,4}', str2parse, flags = regex.LOCALE | regex.UNICODE)[0])
 
 	matches = []
 
@@ -622,7 +622,7 @@ def __numbers_only(str_timestamp):
 
 	return matches
 #---------------------------------------------------------------------------
-def __single_dot(str_timestamp):
+def __single_dot(str2parse):
 	"""Expand fragments containing a single dot.
 
 	Standard colloquial date format in Germany: day.month.year
@@ -631,10 +631,10 @@ def __single_dot(str_timestamp):
 		- 14th current month this year
 		- 14th next month this year
 	"""
-	if not regex.match("^(\s|\t)*\d{1,2}\.{1}(\s|\t)*$", str_timestamp):
+	if not regex.match(u"^(\s|\t)*\d{1,2}\.{1}(\s|\t)*$", str2parse, flags = regex.LOCALE | regex.UNICODE):
 		return []
 
-	val = int(regex.findall('\d+', str_timestamp)[0])
+	val = int(regex.findall(u'\d+', str2parse, flags = regex.LOCALE | regex.UNICODE)[0])
 	now = mxDT.now()
 	enc = gmI18N.get_encoding()
 
@@ -666,7 +666,7 @@ def __single_dot(str_timestamp):
 
 	return matches
 #---------------------------------------------------------------------------
-def str2fuzzy_timestamp_matches(str_timestamp=None, default_time=None):
+def str2fuzzy_timestamp_matches(str2parse=None, default_time=None):
 	"""
 	Turn a string into candidate fuzzy timestamps and auto-completions the user is likely to type.
 
@@ -678,22 +678,22 @@ def str2fuzzy_timestamp_matches(str_timestamp=None, default_time=None):
 		this value will be used
 	@type default_time: an mx.DateTime.DateTimeDelta instance
 	"""
-	matches = __single_dot(str_timestamp)
-	matches.extend(__numbers_only(str_timestamp))
-	matches.extend(__single_slash(str_timestamp))
-	matches.extend(__single_char(str_timestamp))
-	matches.extend(__explicit_offset(str_timestamp))
+	matches = __single_dot(str2parse)
+	matches.extend(__numbers_only(str2parse))
+	matches.extend(__single_slash(str2parse))
+	matches.extend(__single_char(str2parse))
+	matches.extend(__explicit_offset(str2parse))
 
 	# try mxDT parsers
 	if mxDT is not None:
 		try:
 			# date ?
 			date_only = mxDT.Parser.DateFromString (
-				text = str_timestamp,
+				text = str2parse,
 				formats = ('euro', 'iso', 'us', 'altus', 'altiso', 'lit', 'altlit', 'eurlit')
 			)
 			# time, too ?
-			time = mxDT.Parser.TimeFromString(text = str_timestamp)
+			time = mxDT.Parser.TimeFromString(text = str2parse)
 			datetime = date_only + time
 			if datetime == date_only:
 				accuracy = acc_days
@@ -822,7 +822,14 @@ class cFuzzyTimestamp:
 		return self.timestamp
 	#-----------------------------------------------------------------------
 	def get_pydt(self):
-		tz = cFixedOffsetTimezone(self.timestamp.gmtoffset().minutes, self.timestamp.tz)
+		try:
+			gmtoffset = self.timestamp.gmtoffset()
+		except mxDT.Error:
+			# Windows cannot deal with dates < 1970, so
+			# when that happens switch to now()
+			now = mxDT.now()
+			gmtoffset = now.gmtoffset()
+		tz = cFixedOffsetTimezone(gmtoffset.minutes, self.timestamp.tz)
 		secs, msecs = divmod(self.timestamp.second, 1)
 		ts = pyDT.datetime (
 			year = self.timestamp.year,
@@ -871,12 +878,12 @@ if __name__ == '__main__':
 		val = None
 		while val != 'exit':
 			val = raw_input('Enter date fragment ("exit" quits): ')
-			matches = str2fuzzy_timestamp_matches(str_timestamp = val)
+			matches = str2fuzzy_timestamp_matches(str2parse = val)
 			for match in matches:
-				print match['label']
-				print match['data']
-				print "---------------"
-
+				print 'label shown  :', match['label']
+				print 'data attached:', match['data']
+				print ""
+			print "---------------"	
 	#-------------------------------------------------
 	def test_cFuzzyTimeStamp():
 		print "testing fuzzy timestamp class"
@@ -901,19 +908,44 @@ if __name__ == '__main__':
 			print "  str()            :", str(fts)
 			print "  repr()           :", repr(fts)
 			raw_input('press ENTER to continue')
-
 	#-------------------------------------------------
-	gmI18N.activate_locale()
-	gmI18N.install_domain('gnumed')
+	def test_get_pydt():
+		print "testing platform for handling dates before 1970"
+		print "-----------------------------------------------"
+		ts = mxDT.DateTime(1935, 4, 2)
+		fts = cFuzzyTimestamp(timestamp=ts)
+		print "fts           :", fts
+		print "fts.get_pydt():", fts.get_pydt()
+	#-------------------------------------------------
+	if len(sys.argv) > 0 and sys.argv[1] == "test":
+		gmI18N.activate_locale()
+		gmI18N.install_domain('gnumed')
 
-	init()
+		init()
 
-	test_date_time()
-	test_str2fuzzy_timestamp_matches()
-	test_cFuzzyTimeStamp()
+		#test_date_time()
+		#test_str2fuzzy_timestamp_matches()
+		#test_cFuzzyTimeStamp()
+		test_get_pydt()
 
 #===========================================================================
 # $Log: gmDateTime.py,v $
+# Revision 1.14  2007/09/04 23:28:06  ncq
+# - document what's happening
+#
+# Revision 1.13  2007/09/04 21:59:30  ncq
+# - try to work around Windows breakage before 1970
+#
+# Revision 1.12  2007/09/03 12:56:00  ncq
+# - test for dates before 1970
+#
+# Revision 1.11  2007/06/15 08:10:40  ncq
+# - improve test suite
+#
+# Revision 1.10  2007/06/15 08:01:09  ncq
+# - better argument naming
+# - fix regexen for unicode/locale
+#
 # Revision 1.9  2007/05/21 17:13:12  ncq
 # - import gmI18N
 #
