@@ -8,82 +8,34 @@
 """
 #================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmEMRStructWidgets.py,v $
-# $Id: gmEMRStructWidgets.py,v 1.62.2.1 2007/06/01 15:14:46 ncq Exp $
-__version__ = "$Revision: 1.62.2.1 $"
+# $Id: gmEMRStructWidgets.py,v 1.71 2007/10/07 12:32:41 ncq Exp $
+__version__ = "$Revision: 1.71 $"
 __author__ = "cfmoro1976@yahoo.es, karsten.hilbert@gmx.net"
 __license__ = "GPL"
 
 # stdlib
 import sys, re, datetime as pydt
 
+
 # 3rd party
 import wx
 
+
 # GNUmed
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
 from Gnumed.pycommon import gmLog, gmI18N, gmMatchProvider, gmDispatcher, gmSignals, gmTools, gmDateTime, gmCfg
-from Gnumed.business import gmEMRStructItems, gmPerson, gmSOAPimporter
-if __name__ == '__main__':
-	gmI18N.install_domain()
-from Gnumed.wxpython import gmPhraseWheel, gmGuiHelpers, gmEditArea
+from Gnumed.business import gmEMRStructItems, gmPerson, gmSOAPimporter, gmSurgery
+from Gnumed.wxpython import gmPhraseWheel, gmGuiHelpers, gmListWidgets
 from Gnumed.wxGladeWidgets import wxgIssueSelectionDlg, wxgMoveNarrativeDlg
 from Gnumed.wxGladeWidgets import wxgHealthIssueEditAreaPnl, wxgHealthIssueEditAreaDlg
 from Gnumed.wxGladeWidgets import wxgEncounterEditAreaPnl, wxgEncounterEditAreaDlg
 from Gnumed.wxGladeWidgets import wxgEpisodeEditAreaPnl, wxgEpisodeEditAreaDlg
 
+
 _log = gmLog.gmDefLog
 _log.Log(gmLog.lInfo, __version__)
 	
-#================================================================
-class cMoveNarrativeDlg(wxgMoveNarrativeDlg.wxgMoveNarrativeDlg):
-
-	def __init__(self, *args, **kwargs):
-
-		self.encounter = kwargs['encounter']
-		self.source_episode = kwargs['episode']
-		del kwargs['encounter']
-		del kwargs['episode']
-
-		wxgMoveNarrativeDlg.wxgMoveNarrativeDlg.__init__(self, *args, **kwargs)
-
-		self.LBL_source_episode.SetLabel(u'%s%s' % (self.source_episode['description'], gmTools.coalesce(self.source_episode['health_issue'], u'', u' (%s)')))
-		self.LBL_encounter.SetLabel('%s: %s %s - %s' % (
-			self.encounter['started'].strftime('%x').decode(gmI18N.get_encoding()),
-			self.encounter['l10n_type'],
-			self.encounter['started'].strftime('%H:%M'),
-			self.encounter['last_affirmed'].strftime('%H:%M')
-		))
-		pat = gmPerson.gmCurrentPatient()
-		emr = pat.get_emr()
-		narr = emr.get_clin_narrative(episodes=[self.source_episode['pk_episode']], encounters=[self.encounter['pk_encounter']])
-		if len(narr) == 0:
-			narr = [{'narrative': _('There is no narrative for this episode in this encounter.')}]
-		self.LBL_narrative.SetLabel(u'\n'.join([n['narrative'] for n in narr]))
-
-	#------------------------------------------------------------
-	def _on_move_button_pressed(self, event):
-
-		target_episode = self._PRW_episode_selector.GetData(can_create = False)
-
-		if target_episode is None:
-			gmDispatcher.send(signal=gmSignals.statustext(), msg=_('Must select episode to move narrative to first.'))
-			# FIXME: set to pink
-			self._PRW_episode_selector.SetFocus()
-			return False
-
-		target_episode = gmEMRStructItems.cEpisode(aPK_obj=target_episode)
-
-		self.encounter.transfer_clinical_data (
-			source_episode = self.source_episode,
-			target_episode = target_episode
-		)
-
-		if self.IsModal():
-			self.EndModal(wx.ID_OK)
-		else:
-			self.Close()
-
 #================================================================
 # encounter related widgets/functions
 #----------------------------------------------------------------
@@ -166,8 +118,8 @@ class cEncounterEditAreaPnl(wxgEncounterEditAreaPnl.wxgEncounterEditAreaPnl):
 	#--------------------------------------------------------
 	def __init_ui(self):
 		self._LCTRL_problems.InsertColumn(0, _('Episode'))
-		self._LCTRL_problems.InsertColumn(1, _('Foundational Issue'))
-		self._LCTRL_problems.InsertColumn(2, _('Assessment'))
+#		self._LCTRL_problems.InsertColumn(1, _('Foundational Issue'))
+#		self._LCTRL_problems.InsertColumn(2, _('Assessment'))
 		self._LCTRL_problems.InsertColumn(3, _('Narrative'))
 	#--------------------------------------------------------
 	# external API
@@ -187,10 +139,10 @@ class cEncounterEditAreaPnl(wxgEncounterEditAreaPnl.wxgEncounterEditAreaPnl):
 		self._LCTRL_problems.DeleteAllItems()
 		for episode in episodes:
 			row_num = self._LCTRL_problems.InsertStringItem(pos, label = episode['description'])
-			self._LCTRL_problems.SetStringItem(index = row_num, col = 1, label = gmTools.coalesce(episode['health_issue'], ''))
+#			self._LCTRL_problems.SetStringItem(index = row_num, col = 1, label = gmTools.coalesce(episode['health_issue'], ''))
 		if len(episodes) > 0:
 			self._LCTRL_problems.SetColumnWidth(col=0, width=wx.LIST_AUTOSIZE)
-			self._LCTRL_problems.SetColumnWidth(col=1, width=wx.LIST_AUTOSIZE)		# wx.LIST_AUTOSIZE_USEHEADER
+#			self._LCTRL_problems.SetColumnWidth(col=1, width=wx.LIST_AUTOSIZE)		# wx.LIST_AUTOSIZE_USEHEADER
 
 		self._PRW_encounter_type.SetText(self.__encounter['l10n_type'], data=self.__encounter['pk_type'])
 
@@ -266,7 +218,9 @@ class cEncounterEditAreaDlg(wxgEncounterEditAreaDlg.wxgEncounterEditAreaDlg):
 
 		self._PNL_edit_area.refresh(encounter=encounter)
 
-		self.Refresh()		# needed ?
+#		self.Refresh()		# needed ?
+
+		self.Fit()
 	#--------------------------------------------------------
 	def _on_save_button_pressed(self, evt):
 		if self._PNL_edit_area.save():
@@ -303,12 +257,12 @@ def move_episode_to_issue(episode=None, target_issue=None, save_to_backend=False
 	db_cfg = gmCfg.cCfgSQL()
 	epi_ttl = int(db_cfg.get2 (
 		option = u'episode.ttl',
-		workplace = gmPerson.gmCurrentProvider().workplace,
+		workplace = gmSurgery.gmCurrentPractice().active_workplace,
 		bias = 'user',
 		default = 60				# 2 months
 	))
 	if target_issue.close_expired_episode(ttl=epi_ttl) is True:
-		gmDispatcher.send(signal=gmSignals.statustext(), msg=_('Closed episodes older than %s days on health issue [%s]') % (epi_ttl, target_issue['description']))
+		gmDispatcher.send(signal='statustext', msg=_('Closed episodes older than %s days on health issue [%s]') % (epi_ttl, target_issue['description']))
 	existing_epi = target_issue.get_open_episode()
 
 	# no more open episode on target issue: should work now
@@ -381,6 +335,25 @@ def move_episode_to_issue(episode=None, target_issue=None, save_to_backend=False
 		episode.save_payload()
 	return True
 #----------------------------------------------------------------
+class cEpisodeListSelectorDlg(gmListWidgets.cGenericListSelectorDlg):
+
+	# FIXME: support pre-selection
+
+	def __init__(self, *args, **kwargs):
+
+		episodes = kwargs['episodes']
+		del kwargs['episodes']
+
+		gmListWidgets.cGenericListSelectorDlg.__init__(self, *args, **kwargs)
+
+		self.SetTitle(_('Select the episodes you are interested in ...'))
+		self._LCTRL_items.set_columns([_('Episode'), _('Status'), _('Health Issue')])
+		self._LCTRL_items.set_string_items (
+			items = [ [epi['description'], gmTools.bool2str(epi['episode_open'], _('ongoing'), u''), gmTools.coalesce(epi['health_issue'], u'')] for epi in episodes ]
+		)
+		self._LCTRL_items.set_column_widths()
+		self._LCTRL_items.set_data(data = episodes)
+#----------------------------------------------------------------
 class cEpisodeDescriptionPhraseWheel(gmPhraseWheel.cPhraseWheel):
 	"""Let user select an episode *description*.
 
@@ -400,6 +373,7 @@ class cEpisodeDescriptionPhraseWheel(gmPhraseWheel.cPhraseWheel):
 			queries = [u"""
 				select distinct on (description) description, description, 1
 				from clin.episode
+				where description %(fragment_condition)s
 				order by description
 				limit 30"""
 			]
@@ -572,7 +546,7 @@ class cEpisodeEditAreaPnl(wxgEpisodeEditAreaPnl.wxgEpisodeEditAreaPnl):
 			self.__episode['pk_health_issue'] = self._PRW_issue.GetData(can_create=True)
 			issue = gmEMRStructItems.cHealthIssue(aPK_obj=self.__episode['pk_health_issue'])
 			if not move_episode_to_issue(episode = self.__episode, target_issue = issue, save_to_backend = False):
-				gmDispatcher.send(signal=gmSignals.statustext(), msg=
+				gmDispatcher.send(signal='statustext', msg=
 						_('Cannot attach episode [%s] to health issue [%s] because it already has a running episode.') % (
 						self.__episode['description'],
 						issue['description']
@@ -612,6 +586,38 @@ class cEpisodeEditAreaDlg(wxgEpisodeEditAreaDlg.wxgEpisodeEditAreaDlg):
 #================================================================
 # foundational issue related widgets/functions
 #----------------------------------------------------------------
+class cIssueListSelectorDlg(gmListWidgets.cGenericListSelectorDlg):
+
+	# FIXME: support pre-selection
+
+	def __init__(self, *args, **kwargs):
+
+		issues = kwargs['issues']
+		del kwargs['issues']
+
+		gmListWidgets.cGenericListSelectorDlg.__init__(self, *args, **kwargs)
+
+		self.SetTitle(_('Select the health issues you are interested in ...'))
+		self._LCTRL_items.set_columns([u'', _('Health Issue'), u'', u'', u''])
+
+		for issue in issues:
+			if issue['is_confidential']:
+				row_num = self._LCTRL_items.InsertStringItem(sys.maxint, label = _('confidential'))
+				self._LCTRL_items.SetItemTextColour(row_num, col=wx.NamedColour('RED'))
+			else:
+				row_num = self._LCTRL_items.InsertStringItem(sys.maxint, label = u'')
+
+			self._LCTRL_items.SetStringItem(index = row_num, col = 1, label = issue['description'])
+			if issue['clinically_relevant']:
+				self._LCTRL_items.SetStringItem(index = row_num, col = 2, label = _('relevant'))
+			if issue['is_active']:
+				self._LCTRL_items.SetStringItem(index = row_num, col = 3, label = _('active'))
+			if issue['is_cause_of_death']:
+				self._LCTRL_items.SetStringItem(index = row_num, col = 4, label = _('fatal'))
+
+		self._LCTRL_items.set_column_widths()
+		self._LCTRL_items.set_data(data = issues)
+#----------------------------------------------------------------
 class cIssueSelectionPhraseWheel(gmPhraseWheel.cPhraseWheel):
 	"""Let the user select a health issue.
 
@@ -785,9 +791,9 @@ class cHealthIssueEditAreaPnl(wxgHealthIssueEditAreaPnl.wxgHealthIssueEditAreaPn
 		max_age = pydt.datetime.now(tz=pat['dob'].tzinfo) - pat['dob']
 
 		if age is None:
-			gmDispatcher.send(signal=gmSignals.statustext(), msg=_('Cannot parse [%s] into valid interval.') % str_age)
+			gmDispatcher.send(signal='statustext', msg=_('Cannot parse [%s] into valid interval.') % str_age)
 		if age >= max_age:
-			gmDispatcher.send(signal=gmSignals.statustext(), msg=_('Patient is only %s old. Cannot accept age [%s].') % (pat.get_medical_age(), age))
+			gmDispatcher.send(signal='statustext', msg=_('Patient is only %s old. Cannot accept age [%s].') % (pat.get_medical_age(), age))
 
 		if (age is None) or (age >= max_age):
 			self._PRW_age_noted.SetBackgroundColour('pink')
@@ -823,7 +829,7 @@ class cHealthIssueEditAreaPnl(wxgHealthIssueEditAreaPnl.wxgHealthIssueEditAreaPn
 		year_noted = year_noted.get_pydt()
 
 		if year_noted >= pydt.datetime.now(tz=year_noted.tzinfo):
-			gmDispatcher.send(signal=gmSignals.statustext(), msg=_('Condition diagnosed in the future.'))
+			gmDispatcher.send(signal='statustext', msg=_('Condition diagnosed in the future.'))
 			self._PRW_year_noted.SetBackgroundColour('pink')
 			self._PRW_year_noted.Refresh()
 			wx.CallAfter(self._PRW_age_noted.SetText, '')
@@ -948,7 +954,7 @@ class cHealthIssueEditAreaPnl(wxgHealthIssueEditAreaPnl.wxgHealthIssueEditAreaPn
 
 		if self.__issue is None:
 			if not can_create:
-				gmDispatcher.send(signal=gmSignals.statustext, msg=_('Creating new health issue not allowed.'))
+				gmDispatcher.send(signal='statustext', msg=_('Creating new health issue not allowed.'))
 				return False
 			pat = gmPerson.gmCurrentPatient()
 			success, self.__issue = gmEMRStructItems.create_health_issue (
@@ -1025,6 +1031,10 @@ class cHealthIssueEditAreaDlg(wxgHealthIssueEditAreaDlg.wxgHealthIssueEditAreaDl
 if __name__ == '__main__':
 
 	_log.SetAllLogLevels(gmLog.lData)
+
+	gmI18N.activate_locale()
+	gmI18N.install_domain()
+	gmDateTime.init()
 	
 	#================================================================	
 	class testapp (wx.App):
@@ -1151,8 +1161,33 @@ if __name__ == '__main__':
 
 #================================================================
 # $Log: gmEMRStructWidgets.py,v $
-# Revision 1.62.2.1  2007/06/01 15:14:46  ncq
-# - episodes only have pk_episode, not pk_encounter
+# Revision 1.71  2007/10/07 12:32:41  ncq
+# - workplace property now on gmSurgery.gmCurrentPractice() borg
+#
+# Revision 1.70  2007/08/29 22:08:57  ncq
+# - narrative widgets factored out
+#
+# Revision 1.69  2007/08/29 14:38:39  ncq
+# - improve encounter details dialog
+#
+# Revision 1.68  2007/08/15 09:19:32  ncq
+# - cleanup
+#
+# Revision 1.67  2007/08/13 22:00:48  ncq
+# - proper year format specs
+#
+# Revision 1.66  2007/08/13 11:07:41  ncq
+# - make episode descriptions phrasewheel actually *match*
+#   on input, IOW, add a where clause to the select ;-)
+#
+# Revision 1.65  2007/08/12 00:09:07  ncq
+# - no more gmSignals.py
+#
+# Revision 1.64  2007/07/13 12:20:48  ncq
+# - select_narrative_from_episodes(), related widgets, and test suite
+#
+# Revision 1.63  2007/06/10 10:02:53  ncq
+# - episode pk is pk_episode
 #
 # Revision 1.62  2007/05/18 13:28:57  ncq
 # - implement cMoveNarrativeDlg
