@@ -1,4 +1,4 @@
-"""GnuMed GUI helper classes and functions
+"""GNUmed GUI helper classes and functions.
 
 This module provides some convenient wxPython GUI
 helper thingies that are widely used throughout
@@ -11,21 +11,23 @@ to anybody else.
 """
 # ========================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmGuiHelpers.py,v $
-# $Id: gmGuiHelpers.py,v 1.61.2.1 2007/06/06 08:21:39 ncq Exp $
-__version__ = "$Revision: 1.61.2.1 $"
+# $Id: gmGuiHelpers.py,v 1.76.2.1 2007/12/24 23:03:23 shilbert Exp $
+__version__ = "$Revision: 1.76.2.1 $"
 __author__  = "K. Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL (details at http://www.gnu.org)"
 
-import sys, os, shutil, datetime as pyDT, traceback
-
+import sys, os, shutil, datetime as pyDT, traceback, exceptions, re as regex, codecs
 if __name__ == '__main__':
 	sys.exit("This is not intended to be run standalone !")
 
+
 import wx
 
+
 from Gnumed.business import gmSurgery
-from Gnumed.pycommon import gmLog, gmGuiBroker, gmPG2, gmLoginInfo, gmDispatcher, gmSignals
-from Gnumed.wxGladeWidgets import wxg3ButtonQuestionDlg, wxg2ButtonQuestionDlg, wxgUnhandledExceptionDlg
+from Gnumed.pycommon import gmLog, gmGuiBroker, gmPG2, gmLoginInfo, gmDispatcher, gmSignals, gmTools, gmCfg, gmI18N
+from Gnumed.wxGladeWidgets import wxg3ButtonQuestionDlg, wxg2ButtonQuestionDlg, wxgUnhandledExceptionDlg, wxgGreetingEditorDlg
+
 
 _log = gmLog.gmDefLog
 _log.Log(gmLog.lData, __version__)
@@ -37,32 +39,59 @@ def handle_uncaught_exception_wx(t, v, tb):
 
 	_log.LogException('unhandled exception caught', (t,v,tb), verbose=True)
 
-	for target in _log.get_targets():
-		if not isinstance(target, gmLog.cLogTargetFile):
-			continue
-		name = os.path.basename(target.ID)
-		name, ext = os.path.splitext(name)
-		new_name = os.path.expanduser(os.path.join (
-			'~',
-			'gnumed',
-			'logs',
-			'%s_%s%s' % (name, pyDT.datetime.now().strftime('%Y-%m-%d_%H-%M-%S'), ext)
-		))
-		_log.Log(gmLog.lWarn, 'syncing log file for backup to [%s]' % new_name)
-		_log.flush()
-		shutil.copy2(target.ID, new_name)
-
+	# careful: MSW does reference counting on Begin/End* :-(
 	try: wx.EndBusyCursor()
 	except: pass
 
-	dlg = cUnhandledExceptionDlg(parent = None, id = -1, exception = (t, v, tb), logfile = new_name)
-	dlg.ShowModal()
-	dlg.Destroy()
+	if t == exceptions.ImportError:
+		gm_show_error (
+			aTitle = _('Missing GNUmed module'),
+			aMessage = _(
+				'GNUmed detected that parts of it are not\n'
+				'properly insalled. The following message\n'
+				'names the missing part:\n'
+				'\n'
+				' "%s"\n'
+				'\n'
+				'Please make sure to get the missing\n'
+				'parts installed. Otherwise some of the\n'
+				'functionality will not be accessible.'
+			) % v
+		)
+	else:
+		for target in _log.get_targets():
+			if not isinstance(target, gmLog.cLogTargetFile):
+				continue
+			original_name = target.ID
+			name = os.path.basename(target.ID)
+			name, ext = os.path.splitext(name)
+			new_name = os.path.expanduser(os.path.join (
+				'~',
+				'gnumed',
+				'logs',
+				'%s_%s%s' % (name, pyDT.datetime.now().strftime('%Y-%m-%d_%H-%M-%S'), ext)
+			))
+			_log.Log(gmLog.lWarn, 'syncing log file for backup to [%s]' % new_name)
+			_log.flush()
+			shutil.copy2(original_name, new_name)
+
+		dlg = cUnhandledExceptionDlg(parent = None, id = -1, exception = (t, v, tb), logfile = new_name)
+		dlg.ShowModal()
+		comment = dlg._TCTRL_comment.GetValue()
+		if comment is not None:
+			_log.Log(gmLog.lErr, u'user comment: %s' % comment.strip())
+			_log.flush()
+		dlg.Destroy()
 # ------------------------------------------------------------------------
 def install_wx_exception_handler():
+	office = gmSurgery.gmCurrentPractice()
+	global helpdesk
+	helpdesk = office.helpdesk
+
 	global _prev_excepthook
 	_prev_excepthook = sys.excepthook
 	sys.excepthook = handle_uncaught_exception_wx
+
 	return True
 # ------------------------------------------------------------------------
 def uninstall_wx_exception_handler():
@@ -78,14 +107,13 @@ class cUnhandledExceptionDlg(wxgUnhandledExceptionDlg.wxgUnhandledExceptionDlg):
 
 		exception = kwargs['exception']
 		del kwargs['exception']
-		logfile = kwargs['logfile']
+		self.logfile = kwargs['logfile']
 		del kwargs['logfile']
 
 		wxgUnhandledExceptionDlg.wxgUnhandledExceptionDlg.__init__(self, *args, **kwargs)
 
-		office = gmSurgery.gmCurrentPractice()
-		self._TCTRL_helpdesk.SetValue(office.helpdesk)
-		self._TCTRL_logfile.SetValue(logfile)
+		self._TCTRL_helpdesk.SetValue(helpdesk)
+		self._TCTRL_logfile.SetValue(self.logfile)
 		t, v, tb = exception
 		self._TCTRL_exc_type.SetValue(str(t))
 		self._TCTRL_exc_value.SetValue(str(v))
@@ -97,6 +125,202 @@ class cUnhandledExceptionDlg(wxgUnhandledExceptionDlg.wxgUnhandledExceptionDlg):
 		top_win = wx.GetApp().GetTopWindow()
 		wx.CallAfter(top_win.Close)
 		evt.Skip()
+	#------------------------------------------
+	def _on_mail_button_pressed(self, evt):
+		receivers = regex.findall (
+			'[\S]+@[\S]+',
+			self._TCTRL_helpdesk.GetValue().strip(),
+			flags = regex.UNICODE | regex.LOCALE
+		)
+		if len(receivers) == 0:
+			receivers = [u'gnumed-devel@gnu.org']
+
+		receiver_string = wx.GetTextFromUser (
+			message = _(
+				'Edit the list of email addresses to send the\n'
+				'bug report to (separate addresses by spaces).\n'
+				'\n'
+				'Note that <gnumed-devel@gnu.org> refers to\n'
+				'the public GNUmed mailing list.'
+			),
+			caption = _('Sending bug report'),
+			default_value = ','.join(receivers),
+			parent = self
+		)
+		if receiver_string.strip() == u'':
+			evt.Skip()
+			return
+
+		receivers = regex.findall (
+			'[\S]+@[\S]+',
+			receiver_string,
+			flags = regex.UNICODE | regex.LOCALE
+		)
+
+		dlg = c2ButtonQuestionDlg (
+			self,
+			-1,
+			caption = _('Sending bug report'),
+			question = _(
+				'Your bug report will be sent to:\n'
+				'\n'
+				'%s\n'
+				'\n'
+				'Make sure you have reviewed the log file for potentially\n'
+				'sensitive information before sending out the bug report.\n'
+				'\n'
+				'Note that emailing the report may take a while depending\n'
+				'on the speed of your internet connection.\n'
+			) % u'\n'.join(receivers),
+			button_defs = [
+				{'label': _('Send report'), 'tooltip': _('Yes, send the bug report.')},
+				{'label': _('Cancel'), 'tooltip': _('No, do not send the bug report.')}
+			],
+			show_checkbox = True,
+			checkbox_msg = _('include log file in bug report')
+		)
+		dlg._CHBOX_dont_ask_again.SetValue(True)
+		go_ahead = dlg.ShowModal()
+		if go_ahead == wx.ID_NO:
+			dlg.Destroy()
+			evt.Skip()
+			return
+
+		comment = self._TCTRL_comment.GetValue().strip()
+		if comment == u'':
+			comment = wx.GetTextFromUser (
+				message = _(
+					'Please enter a short note on what you\n'
+					'were about to do in GNUmed:'
+				),
+				caption = _('Sending bug report'),
+				parent = self
+			)
+			if comment.strip() == u'':
+				dlg.Destroy()
+				evt.Skip()
+				return
+
+		msg = u"Report sent via GNUmed's handler for unexpected exceptions:\n\n %s\n\n" % comment
+		if dlg._CHBOX_dont_ask_again.GetValue():
+			for line in codecs.open(self.logfile, 'rU', 'latin1', 'replace'):
+				msg = msg + line
+
+		dlg.Destroy()
+
+		wx.BeginBusyCursor()
+		gmTools.send_mail (
+			sender = gmTools.default_mail_sender,
+			receiver = receivers,
+			subject = u'<bug>: %s' % comment,
+			message = msg,
+			encoding = gmI18N.get_encoding(),
+			server = gmTools.default_mail_server,
+			auth = {'user': gmTools.default_mail_sender, 'password': u'gnumed-at-gmx-net'}
+		)
+		wx.EndBusyCursor()
+
+		evt.Skip()
+	#------------------------------------------
+	def _on_view_log_button_pressed(self, evt):
+		from Gnumed.pycommon import gmMimeLib
+		gmMimeLib.call_viewer_on_file(self.logfile, block = False)
+		evt.Skip()
+# ========================================================================
+def configure_string_option(parent=None, message=None, option=None, bias=u'user', default_value=u'', validator=None):
+
+	dbcfg = gmCfg.cCfgSQL()
+
+	current_value = dbcfg.get2 (
+		option = option,
+		workplace = gmSurgery.gmCurrentPractice().active_workplace,
+		bias = bias,
+		default = default_value
+	)
+
+	if parent is None:
+		parent = wx.GetApp().GetTopWindow()
+
+	while True:
+		dlg = wx.TextEntryDialog (
+			parent = parent,
+			message = message,
+			caption = _('Configuration'),
+			defaultValue = u'%s' % current_value,
+			style = wx.OK | wx.CANCEL | wx.CENTRE
+		)
+		result = dlg.ShowModal()
+		if result == wx.ID_CANCEL:
+			dlg.Destroy()
+			return
+
+		user_val = dlg.GetValue().strip()
+		dlg.Destroy()
+
+		if user_val == current_value:
+			return
+
+		validated, user_val = validator(user_val)
+		if validated:
+			break
+
+		gmDispatcher.send (
+			signal = u'statustext',
+			msg = _('Value [%s] not valid for option <%s>.') % (user_val, option),
+			beep = True
+		)
+
+	dbcfg = gmCfg.cCfgSQL()
+	dbcfg.set (
+		workplace = gmSurgery.gmCurrentPractice().active_workplace,
+		option = option,
+		value = user_val
+	)
+
+	return
+# ========================================================================
+def configure_boolean_option(parent=None, question=None, option=None, button_tooltips=None):
+
+	if parent is None:
+		parent = wx.GetApp().GetTopWindow()
+
+	tooltips = [
+		_('Set "%s" to <True>.') % option,
+		_('Set "%s" to <False>.') % option,
+		_('Abort the dialog and do not change the current setting.')
+	]
+	if button_tooltips is not None:
+		for idx in range(len(button_tooltips)):
+			tooltips[idx] = button_tooltips[idx]
+
+	dlg = c3ButtonQuestionDlg (
+		parent,
+		-1,
+		caption = _('Configuration'),
+		question = question,
+		button_defs = [
+			{'label': _('Yes'), 'tooltip': tooltips[0]},
+			{'label': _('No'), 'tooltip': tooltips[1]},
+			{'label': _('Cancel'), 'tooltip': tooltips[2], 'default': True}
+		]
+	)
+
+	decision = dlg.ShowModal()
+	dbcfg = gmCfg.cCfgSQL()
+	if decision == wx.ID_YES:
+		dbcfg.set (
+			workplace = gmSurgery.gmCurrentPractice().active_workplace,
+			option = option,
+			value = True
+		)
+	elif decision == wx.ID_NO:
+		dbcfg.set (
+			workplace = gmSurgery.gmCurrentPractice().active_workplace,
+			option = option,
+			value = False
+		)
+
+	return
 # ========================================================================
 class c2ButtonQuestionDlg(wxg2ButtonQuestionDlg.wxg2ButtonQuestionDlg):
 
@@ -111,6 +335,12 @@ class c2ButtonQuestionDlg(wxg2ButtonQuestionDlg.wxg2ButtonQuestionDlg):
 		except KeyError:
 			show_checkbox = False
 
+		try:
+			checkbox_msg = kwargs['checkbox_msg']
+			del kwargs['checkbox_msg']
+		except KeyError:
+			checkbox_msg = None
+
 		del kwargs['caption']
 		del kwargs['question']
 		del kwargs['button_defs']
@@ -122,6 +352,9 @@ class c2ButtonQuestionDlg(wxg2ButtonQuestionDlg.wxg2ButtonQuestionDlg):
 
 		if not show_checkbox:
 			self._CHBOX_dont_ask_again.Hide()
+		else:
+			if checkbox_msg is not None:
+				self._CHBOX_dont_ask_again.SetLabel(checkbox_msg)
 
 		buttons = [self._BTN_1, self._BTN_2]
 		for idx in range(len(button_defs)):
@@ -170,8 +403,11 @@ class c3ButtonQuestionDlg(wxg3ButtonQuestionDlg.wxg3ButtonQuestionDlg):
 		for idx in range(len(button_defs)):
 			buttons[idx].SetLabel(label = button_defs[idx]['label'])
 			buttons[idx].SetToolTipString(button_defs[idx]['tooltip'])
-			if button_defs[idx]['default']:
-				buttons[idx].SetDefault()
+			try:
+				if button_defs[idx]['default']:
+					buttons[idx].SetDefault()
+			except KeyError:
+				pass
 
 		self.Fit()
 	#--------------------------------------------------------
@@ -188,6 +424,72 @@ class c3ButtonQuestionDlg(wxg3ButtonQuestionDlg.wxg3ButtonQuestionDlg):
 			self.EndModal(wx.ID_NO)
 		else:
 			self.Close()
+
+# ========================================================================
+class cStartupProgressBar(wx.ProgressDialog):
+	def __init__(self, nr_actions):
+
+		wx.ProgressDialog.__init__(
+			self,
+			parent = None,
+			title = _('GNUmed: Starting up ...'),
+			message = u' ' * 80 + u'\n\n\n',
+			maximum = nr_actions,
+			style = wx.PD_ELAPSED_TIME | wx.PD_CAN_ABORT
+		)
+
+		# set window icon
+		paths = gmTools.gmPaths(app_name = u'gnumed', wx = wx)
+		png_fname = os.path.join(paths.system_app_data_dir, 'bitmaps', 'serpent.png')
+		icon = wx.EmptyIcon()
+		try:
+			icon.LoadFile(png_fname, wx.BITMAP_TYPE_PNG)
+		except:
+			_log.Log(gmLog.lWarn, 'wx.Icon.LoadFile() not supported')
+		self.SetIcon(icon)
+		self.idx = 0
+#		self.nr_plugins = nr_plugins
+#		self.prev_plugin = ""
+	#----------------------------------------------------------
+	def Update (self, msg = None):
+#		if result == -1:
+#			result = ""
+#		elif result == 0:
+#			result = _("failed")
+#		else:
+#			result = _("success")
+
+#_("GNUmed: configuring [%s] (%s plugins)") % (gmSurgery.gmCurrentPractice().active_workplace, nr_plugins),
+#_("loading list of plugins                               "),
+		wx.ProgressDialog.Update(self, self.idx, msg)
+#		self.prev_plugin = plugin
+		self.idx += 1
+
+#			_("previous: %s (%s)\ncurrent (%s/%s): %s") % (
+#				self.prev_plugin,
+#				result,
+#				(self.idx+1),
+#				self.nr_plugins,
+#				plugin
+#			)
+
+# ========================================================================
+class cGreetingEditorDlg(wxgGreetingEditorDlg.wxgGreetingEditorDlg):
+
+	def __init__(self, *args, **kwargs):
+		wxgGreetingEditorDlg.wxgGreetingEditorDlg.__init__(self, *args, **kwargs)
+
+		self.surgery = gmSurgery.gmCurrentPractice()
+		self._TCTRL_message.SetValue(self.surgery.db_logon_banner)
+	#--------------------------------------------------------
+	# event handlers
+	#--------------------------------------------------------
+	def _on_save_button_pressed(self, evt):
+		self.surgery.db_logon_banner = self._TCTRL_message.GetValue().strip()
+		if self.IsModal():
+			self.EndModal(wx.ID_SAVE)
+		else:
+			self.Close()
 # ========================================================================
 class cTreeExpansionHistoryMixin:
 	"""TreeCtrl mixin class to record expansion history."""
@@ -293,32 +595,6 @@ class cFileDropTarget(wx.FileDropTarget):
 	def OnDropFiles(self, x, y, filenames):
 		self.target.add_filenames(filenames)
 # ========================================================================
-def gm_SingleChoiceDialog(aMessage = None, aTitle = None, aLogLevel = None, choices = None):
-    if aMessage is None:
-        aMessage = _('programmer forgot to specify info message')
-
-    if aLogLevel is not None:
-        log_msg = aMessage.replace('\015', ' ').replace('\012', ' ')
-        _log.Log(aLogLevel, log_msg)
-
-    if aTitle is None:
-        aTitle = _('generic single choice dialog')
-
-    dlg = wx.SingleChoiceDialog (
-        parent = None,
-        message = aMessage,
-        caption = aTitle,
-        choices = choices,
-        style = wx.OK | wx.CANCEL | wx.CENTRE
-    )
-    btn_pressed = dlg.ShowModal()
-    dlg.Destroy()
-
-    if btn_pressed == wx.ID_OK:
-        return dlg.GetSelection()
-    else:
-        return False
-#-------------------------------------------------------------------------
 def gm_show_error(aMessage = None, aTitle = None, aLogLevel = None):
 	if aMessage is None:
 		aMessage = _('programmer forgot to specify error message')
@@ -332,11 +608,6 @@ def gm_show_error(aMessage = None, aTitle = None, aLogLevel = None):
 	if aTitle is None:
 		aTitle = _('generic error message')
 
-	print "-" * len(aTitle)
-	print aTitle
-	print "-" * len(aTitle)
-	print aMessage
-
 	dlg = wx.MessageDialog (
 		parent = None,
 		message = aMessage,
@@ -410,43 +681,6 @@ def gm_show_question(aMessage = 'programmer forgot to specify question', aTitle
 		return False
 	else:
 		return None
-#-------------------------------------------------------------------------
-def get_dbowner_connection(procedure=None, dbo_password=None):
-	if procedure is None:
-		procedure = _('<restricted procedure>')
-
-	# 1) get password for gm-dbo
-	if dbo_password is None:
-		pwd_gm_dbo = wx.GetPasswordFromUser (
-			message = _("""
- [%s]
-
-This is a restricted procedure. We need the
-password for the GNUmed database owner.
-
-Please enter the password for <gm-dbo>:""") % procedure,
-			caption = procedure
-		)
-		if pwd_gm_dbo == '':
-			return None
-	else:
-		pwd_gm_dbo = dbo_password
-
-	# 2) connect as gm-dbo
-	login = gmPG2.get_default_login()
-	dsn = gmPG2.make_psycopg2_dsn(database=login.database, host=login.host, port=login.port, user='gm-dbo', password=pwd_gm_dbo)
-	try:
-		conn = gmPG2.get_connection(dsn=dsn, readonly=False, verbose=True, pooled=False)
-	except:
-		_log.LogException('cannot connect')
-		gm_show_error (
-			aMessage = _('Cannot connect as the GNUmed database owner <gm-dbo>.'),
-			aTitle = procedure,
-			aLogLevel = gmLog.lErr
-		)
-		return None
-
-	return conn
 #----------------------------------------------------------------------
 def makePageTitle(wizPg, title):
 	"""
@@ -572,8 +806,55 @@ class cTextWidgetValidator(wx.PyValidator):
 
 # ========================================================================
 # $Log: gmGuiHelpers.py,v $
-# Revision 1.61.2.1  2007/06/06 08:21:39  ncq
-# - MSW does ref counting on Begin/EndBusyCursor :-( so deal with that
+# Revision 1.76.2.1  2007/12/24 23:03:23  shilbert
+# - removed the dlg.setfocus stuff
+#
+# Revision 1.76  2007/12/04 17:32:33  ncq
+# - improved wording
+#
+# Revision 1.75  2007/12/04 17:08:14  ncq
+# - allow editing bug report targets before sending
+#
+# Revision 1.74  2007/12/04 16:15:28  ncq
+# - remove get_dbowner_connection()
+#
+# Revision 1.73  2007/12/04 15:17:39  ncq
+# - start general purpose progress bar
+#
+# Revision 1.72  2007/11/21 13:31:53  ncq
+# - use send_mail() in exception handling dialog
+#
+# Revision 1.71  2007/10/21 20:18:32  ncq
+# - configure_string_option()
+#
+# Revision 1.70  2007/10/19 12:50:31  ncq
+# - add configure_boolean_option()
+#
+# Revision 1.69  2007/10/11 12:01:51  ncq
+# - make c3ButtonQuestionDlg() more robust in the face of no-default button def
+#
+# Revision 1.68  2007/09/25 20:44:23  ncq
+# - support saving user comment in log file rescued on error
+#
+# Revision 1.67  2007/09/20 21:30:06  ncq
+# - cGreetingEditorDlg
+#
+# Revision 1.66  2007/09/03 11:03:20  ncq
+# - teach top level wx exception handler about ImportError
+#
+# Revision 1.65  2007/08/20 14:25:16  ncq
+# - factor out bits of code out of the actual top level
+#   exception handler in a bid to make it more resilient
+#
+# Revision 1.64  2007/07/18 14:43:01  ncq
+# - do away with accessing console as it often breaks
+#
+# Revision 1.63  2007/06/18 20:31:58  ncq
+# - gm_Multi/SingleChoiceDlg moved to gmListWidgets
+#
+# Revision 1.62  2007/06/11 20:35:06  ncq
+# - MSW does ref counting in Begin/EndBusyCursor
+# - add gmMultiChoiceDialog
 #
 # Revision 1.61  2007/05/14 10:34:07  ncq
 # - no more gm_statustext()
