@@ -4,20 +4,161 @@ This module implements functions a macro can legally use.
 """
 #=====================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmMacro.py,v $
-__version__ = "$Revision: 1.28 $"
+__version__ = "$Revision: 1.38 $"
 __author__ = "K.Hilbert <karsten.hilbert@gmx.net>"
 
 import sys, time, random, types
 
-import wxversion
+
 import wx
 
-from Gnumed.pycommon import gmLog, gmI18N, gmGuiBroker, gmExceptions
+
+if __name__ == '__main__':
+	sys.path.insert(0, '../../')
+from Gnumed.pycommon import gmLog, gmI18N, gmGuiBroker, gmExceptions, gmBorg, gmTools
 from Gnumed.business import gmPerson
-from Gnumed.wxpython import gmGuiHelpers, gmPlugin, gmPatSearchWidgets
+from Gnumed.wxpython import gmGuiHelpers, gmPlugin, gmPatSearchWidgets, gmNarrativeWidgets
+
 
 _log = gmLog.gmDefLog
 
+
+known_placeholders = [
+	'lastname',
+	'firstname',
+	'title',
+	'date_of_birth',
+	'progress_notes',
+	'soap',
+	'soap_s',
+	'soap_o',
+	'soap_a',
+	'soap_p'
+]
+
+# those must satisfy '.+::.+' when used
+known_variant_placeholders = [
+	'soap',
+	'progress_notes',
+	'date_of_birth'
+]
+
+#=====================================================================
+class gmPlaceholderHandler(gmBorg.cBorg):
+	"""Replaces placeholders in forms, fields, etc.
+
+	- patient related placeholders operate on the currently active patient
+	- is passed to the forms handling code, for example
+
+	Note that this cannot be called from a non-gui thread unless
+	wrapped in wx.CallAfter.
+	"""
+	def __init__(self, *args, **kwargs):
+
+		self.pat = gmPerson.gmCurrentPatient()
+		self.debug = False
+	#--------------------------------------------------------
+	# __getitem__ API
+	#--------------------------------------------------------
+	def __getitem__(self, placeholder):
+		"""Map self['placeholder'] to self.placeholder.
+
+		This is useful for replacing placeholders parsed out
+		of documents as strings.
+
+		Unknown placeholders still deliver a result but it will be
+		made glaringly obvious that the placeholder was unknown.
+		"""
+		# static placeholders
+		if placeholder in known_placeholders:
+			return getattr(self, placeholder)
+
+		# variable placeholders
+		parts = placeholder.split('::', 1)
+		if len(parts) == 2:
+			name, data = parts
+			handler = getattr(self, '_get_variant_%s' % name, None)
+			if handler is not None:
+				return handler(data = data)
+
+		if self.debug:
+			return _('unknown placeholder: <%s>' % placeholder)
+
+		return None
+	#--------------------------------------------------------
+	# properties actually handling placeholders
+	#--------------------------------------------------------
+	# property helpers
+	#--------------------------------------------------------
+	def _setter_noop(self, val):
+		"""This does nothing, used as a NOOP properties setter."""
+		pass
+	#--------------------------------------------------------
+	def _get_lastname(self):
+		return self.pat.get_active_name()['lastnames']
+	#--------------------------------------------------------
+	def _get_firstname(self):
+		return self.pat.get_active_name()['firstnames']
+	#--------------------------------------------------------
+	def _get_title(self):
+		return gmTools.coalesce(self.pat.get_active_name()['title'], u'')
+	#--------------------------------------------------------
+	def _get_dob(self):
+		return self._get_variant_date_of_birth(data='%x')
+	#--------------------------------------------------------
+	def _get_progress_notes(self):
+		return self._get_variant_soap()
+	#--------------------------------------------------------
+	def _get_soap_s(self):
+		return self._get_variant_soap(data = u's')
+	#--------------------------------------------------------
+	def _get_soap_o(self):
+		return self._get_variant_soap(data = u'o')
+	#--------------------------------------------------------
+	def _get_soap_a(self):
+		return self._get_variant_soap(data = u'a')
+	#--------------------------------------------------------
+	def _get_soap_p(self):
+		return self._get_variant_soap(data = u'p')
+	#--------------------------------------------------------
+	def _get_soap_admin(self):
+		return self._get_variant_soap(soap_cats = None)
+	#--------------------------------------------------------
+	# property definitions
+	#--------------------------------------------------------
+	lastname = property(_get_lastname, _setter_noop)
+	firstname = property(_get_firstname, _setter_noop)
+	title = property(_get_title, _setter_noop)
+	date_of_birth = property(_get_dob, _setter_noop)
+
+	progress_notes = property(_get_progress_notes, _setter_noop)
+	soap = property(_get_progress_notes, _setter_noop)
+	soap_s = property(_get_soap_s, _setter_noop)
+	soap_o = property(_get_soap_o, _setter_noop)
+	soap_a = property(_get_soap_a, _setter_noop)
+	soap_p = property(_get_soap_p, _setter_noop)
+	soap_admin = property(_get_soap_admin, _setter_noop)
+	#--------------------------------------------------------
+	# variant handlers
+	#--------------------------------------------------------
+	def _get_variant_progress_notes(self, data=None):
+		return self._get_variant_soap(data=data)
+	#--------------------------------------------------------
+	def _get_variant_soap(self, data=None):
+		if data is not None:
+			data = list(data)
+		narr = gmNarrativeWidgets.select_narrative_from_episodes(soap_cats = data)
+		if len(narr) == 0:
+			return u''
+		narr = [ n['narrative'] for n in narr ]
+		return u'\n'.join(narr)
+	#--------------------------------------------------------
+	def _get_variant_date_of_birth(self, data='%x'):
+		return self.pat['dob'].strftime(data)
+	#--------------------------------------------------------
+	# internal helpers
+	#--------------------------------------------------------
+
 #=====================================================================
 class cMacroPrimitives:
 	"""Functions a macro can legally use.
@@ -73,7 +214,7 @@ class cMacroPrimitives:
 		return 1
 	#-----------------------------------------------------------------
 	def version(self):
-		return "%s $Revision: 1.28 $" % self.__class__.__name__
+		return "%s $Revision: 1.38 $" % self.__class__.__name__
 	#-----------------------------------------------------------------
 	def shutdown_gnumed(self, auth_cookie=None, forced=False):
 		"""Shuts down this client instance."""
@@ -127,7 +268,7 @@ class cMacroPrimitives:
 		if auth_cookie != self.__auth_cookie:
 			_log.Log(gmLog.lErr, 'non-authenticated load_patient_from_external_source()')
 			return (0, _('rejected load_patient_from_external_source(), not authenticated'))
-		if self.__pat.is_locked():
+		if self.__pat.locked:
 			_log.Log(gmLog.lErr, 'patient is locked, cannot load from external source')
 			return (0, _('current patient is locked'))
 		self.__user_done = False
@@ -145,7 +286,7 @@ class cMacroPrimitives:
 		if lock_after_load_cookie != self.__lock_after_load_cookie:
 			_log.Log(gmLog.lWarn, 'patient lock-after-load request rejected due to wrong cookie [%s]' % lock_after_load_cookie)
 			return (0, 'patient lock-after-load request rejected, wrong cookie provided')
-		self.__pat.lock()
+		self.__pat.locked = True
 		self.__pat_lock_cookie = str(random.random())
 		return (1, self.__pat_lock_cookie)
 	#-----------------------------------------------------------------
@@ -155,7 +296,7 @@ class cMacroPrimitives:
 		if auth_cookie != self.__auth_cookie:
 			_log.Log(gmLog.lErr, 'non-authenticated lock_into_patient()')
 			return (0, _('rejected lock_into_patient(), not authenticated'))
-		if self.__pat.is_locked():
+		if self.__pat.locked:
 			_log.Log(gmLog.lErr, 'patient is already locked')
 			return (0, _('already locked into a patient'))
 		searcher = gmPerson.cPatientSearcher_SQL()
@@ -174,7 +315,7 @@ class cMacroPrimitives:
 			return (0, _('several matching patients found for [%s]/%s') % (search_term, search_dict))
 		if not gmPerson.set_active_patient(patient = idents[0]):
 			return (0, _('cannot activate patient [%s] (%s/%s)') % (str(idents[0]), search_term, search_dict))
-		self.__pat.lock()
+		self.__pat.locked = True
 		self.__pat_lock_cookie = str(random.random())
 		return (1, self.__pat_lock_cookie)
 	#-----------------------------------------------------------------
@@ -185,13 +326,13 @@ class cMacroPrimitives:
 			_log.Log(gmLog.lErr, 'non-authenticated unlock_patient()')
 			return (0, _('rejected unlock_patient, not authenticated'))
 		# we ain't locked anyways, so succeed
-		if not self.__pat.is_locked():
+		if not self.__pat.locked:
 			return (1, '')
 		# FIXME: ask user what to do about wrong cookie
 		if unlock_cookie != self.__pat_lock_cookie:
 			_log.Log(gmLog.lWarn, 'patient unlock request rejected due to wrong cookie [%s]' % unlock_cookie)
 			return (0, 'patient unlock request rejected, wrong cookie provided')
-		self.__pat.unlock()
+		self.__pat.locked = False
 		return (1, '')
 	#-----------------------------------------------------------------
 	def assume_staff_identity(self, auth_cookie = None, staff_name = "Dr.Jekyll", staff_creds = None):
@@ -227,7 +368,7 @@ class cMacroPrimitives:
 			self.__user_answer = 0
 		self.__user_done = True
 		if can_break_conn:
-			self.__pat.unlock()
+			self.__pat.locked = False
 			self.__attached = 0
 		return 1
 	#-----------------------------------------------------------------
@@ -239,7 +380,7 @@ class cMacroPrimitives:
 			top_win.Close()
 	#-----------------------------------------------------------------
 	def _load_patient_from_external_source(self):
-		if gmPatSearchWidgets.load_patient_from_external_sources():
+		if gmPatSearchWidgets.load_patient_from_external_sources(search_immediately = True):
 			self.__user_answer = 1
 		else:
 			self.__user_answer = 0
@@ -254,37 +395,100 @@ if __name__ == '__main__':
 	gmI18N.activate_locale()
 	gmI18N.install_domain()
 
-	from Gnumed.pycommon import gmScriptingListener
-	import xmlrpclib
-	listener = gmScriptingListener.cScriptingListener(macro_executor = cMacroPrimitives(personality='unit test'), port=9999)
+	#--------------------------------------------------------
+	def test_placeholders():
+		handler = gmPlaceholderHandler()
+		handler.debug = True
 
-	s = xmlrpclib.ServerProxy('http://localhost:9999')
-	print "should fail:", s.attach()
-	print "should fail:", s.attach('wrong cookie')
-	print "should work:", s.version()
-	print "should fail:", s.raise_gnumed()
-	print "should fail:", s.raise_notebook_plugin('test plugin')
-	print "should fail:", s.lock_into_patient('kirk, james')
-	print "should fail:", s.unlock_patient()
-	status, conn_auth = s.attach('unit test')
-	print "should work:", status, conn_auth
-	print "should work:", s.version()
-	print "should work:", s.raise_gnumed(conn_auth)
-	status, pat_auth = s.lock_into_patient(conn_auth, 'kirk, james')
-	print "should work:", status, pat_auth
-	print "should fail:", s.unlock_patient(conn_auth, 'bogus patient unlock cookie')
-	print "should work", s.unlock_patient(conn_auth, pat_auth)
-	data = {'firstname': 'jame', 'lastnames': 'Kirk', 'gender': 'm'}
-	status, pat_auth = s.lock_into_patient(conn_auth, data)
-	print "should work:", status, pat_auth
-	print "should work", s.unlock_patient(conn_auth, pat_auth)
-	print s.detach('bogus detach cookie')
-	print s.detach(conn_auth)
-	del s
+		for placeholder in ['a', 'b']:
+			print handler[placeholder]
+
+		pat = gmPerson.ask_for_patient()
+		if pat is None:
+			return
+
+		gmPerson.set_active_patient(patient = pat)
+
+		print 'DOB (YYYY-MM-DD):', handler['date_of_birth::%Y-%m-%d']
+
+		app = wx.PyWidgetTester(size = (200, 50))
+		for placeholder in known_placeholders:
+			print placeholder, "=", handler[placeholder]
+
+		ph = 'progress_notes::ap'
+		print '%s: %s' % (ph, handler[ph])
+
+	#--------------------------------------------------------
+	def test_scripting():
+		from Gnumed.pycommon import gmScriptingListener
+		import xmlrpclib
+		listener = gmScriptingListener.cScriptingListener(macro_executor = cMacroPrimitives(personality='unit test'), port=9999)
+
+		s = xmlrpclib.ServerProxy('http://localhost:9999')
+		print "should fail:", s.attach()
+		print "should fail:", s.attach('wrong cookie')
+		print "should work:", s.version()
+		print "should fail:", s.raise_gnumed()
+		print "should fail:", s.raise_notebook_plugin('test plugin')
+		print "should fail:", s.lock_into_patient('kirk, james')
+		print "should fail:", s.unlock_patient()
+		status, conn_auth = s.attach('unit test')
+		print "should work:", status, conn_auth
+		print "should work:", s.version()
+		print "should work:", s.raise_gnumed(conn_auth)
+		status, pat_auth = s.lock_into_patient(conn_auth, 'kirk, james')
+		print "should work:", status, pat_auth
+		print "should fail:", s.unlock_patient(conn_auth, 'bogus patient unlock cookie')
+		print "should work", s.unlock_patient(conn_auth, pat_auth)
+		data = {'firstname': 'jame', 'lastnames': 'Kirk', 'gender': 'm'}
+		status, pat_auth = s.lock_into_patient(conn_auth, data)
+		print "should work:", status, pat_auth
+		print "should work", s.unlock_patient(conn_auth, pat_auth)
+		print s.detach('bogus detach cookie')
+		print s.detach(conn_auth)
+		del s
+
+		listener.shutdown()
+	#--------------------------------------------------------
+
+	if len(sys.argv) > 0 and sys.argv[1] == 'test':
+		test_placeholders()
+		#test_scripting()
 
-	listener.tell_thread_to_stop()
 #=====================================================================
 # $Log: gmMacro.py,v $
+# Revision 1.38  2007/12/03 20:45:16  ncq
+# - add variant placeholder handling ! :-)
+#
+# Revision 1.37  2007/11/05 12:10:21  ncq
+# - support admin soap type
+#
+# Revision 1.36  2007/10/19 12:52:00  ncq
+# - immediately search for patient matches
+#
+# Revision 1.35  2007/09/16 22:40:46  ncq
+# - fix soap_* placeholder handling
+#
+# Revision 1.34  2007/09/09 19:17:44  ncq
+# - add a bunch of placeholders regarding SOAP notes
+#
+# Revision 1.33  2007/08/29 22:09:32  ncq
+# - narrative widgets factored out
+#
+# Revision 1.32  2007/08/13 21:59:54  ncq
+# - add placeholder handler
+# - add progress_notes placeholder
+# - improved test suite
+#
+# Revision 1.31  2007/07/17 21:44:24  ncq
+# - use patient.locked properly
+#
+# Revision 1.30  2007/07/11 21:09:54  ncq
+# - use curr_pat.locked
+#
+# Revision 1.29  2007/07/03 16:00:56  ncq
+# - remove unneeded import
+#
 # Revision 1.28  2007/01/21 12:21:38  ncq
 # - comment on search_dict -> dto
 #
