@@ -2,22 +2,76 @@
 """
 #================================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmMedDocWidgets.py,v $
-# $Id: gmMedDocWidgets.py,v 1.127 2007/05/21 14:49:20 ncq Exp $
-__version__ = "$Revision: 1.127 $"
+# $Id: gmMedDocWidgets.py,v 1.150.2.1 2008/01/08 22:39:31 ncq Exp $
+__version__ = "$Revision: 1.150.2.1 $"
 __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
 
 import os.path, sys, re as regex
 
+
 import wx
 
+
+if __name__ == '__main__':
+	sys.path.insert(0, '../../')
 from Gnumed.pycommon import gmLog, gmI18N, gmCfg, gmPG2, gmMimeLib, gmExceptions, gmMatchProvider, gmDispatcher, gmSignals, gmDateTime, gmTools
-from Gnumed.business import gmPerson, gmMedDoc
-from Gnumed.wxpython import gmGuiHelpers, gmRegetMixin, gmPhraseWheel, gmPlugin
+from Gnumed.business import gmPerson, gmMedDoc, gmEMRStructItems, gmSurgery
+from Gnumed.wxpython import gmGuiHelpers, gmRegetMixin, gmPhraseWheel, gmPlugin, gmEMRStructWidgets, gmListWidgets
 from Gnumed.wxGladeWidgets import wxgScanIdxPnl, wxgReviewDocPartDlg, wxgSelectablySortedDocTreePnl, wxgEditDocumentTypesPnl, wxgEditDocumentTypesDlg
 
+
 _log = gmLog.gmDefLog
 _log.Log(gmLog.lInfo, __version__)
 
+#============================================================
+def _save_file_as_new_document(**kwargs):
+	wx.CallAfter(save_file_as_new_document, **kwargs)
+#----------------------
+def save_file_as_new_document(parent=None, filename=None, document_type=None, unlock_patient=False, **kwargs):
+
+	pat = gmPerson.gmCurrentPatient()
+	if not pat.is_connected():
+		return
+
+	emr = pat.get_emr()
+
+	all_epis = emr.get_episodes()
+	# FIXME: what to do here ? probably create dummy episode
+	if len(all_epis) == 0:
+		epi = emr.add_episode(episode_name = _('Documents'), is_open = False)
+	else:
+		# FIXME: parent=None map to toplevel window
+		dlg = gmEMRStructWidgets.cEpisodeListSelectorDlg(parent = parent, id = -1, episodes = all_epis)
+		dlg.SetTitle(_('Select the episode under which to file the document ...'))
+		btn_pressed = dlg.ShowModal()
+		epi = dlg.get_selected_item_data(only_one = True)
+		dlg.Destroy()
+
+		if btn_pressed == wx.ID_CANCEL:
+			if unlock_patient:
+				pat.locked = False
+			return
+
+	doc_type = gmMedDoc.create_document_type(document_type = document_type)
+
+	docs_folder = pat.get_document_folder()
+	doc = docs_folder.add_document (
+		document_type = doc_type['pk_doc_type'],
+		encounter = emr.get_active_encounter()['pk_encounter'],
+		episode = epi['pk_episode']
+	)
+	part = doc.add_part(file = filename)
+	part['filename'] = filename
+	part.save_payload()
+
+	if unlock_patient:
+		pat.locked = False
+
+	gmDispatcher.send(signal = 'statustext', msg = _('Imported new document from [%s].' % filename), beep = True)
+
+	return
+#----------------------
+gmDispatcher.connect(signal = u'import_document_from_file', receiver = _save_file_as_new_document)
 #============================================================
 class cDocumentCommentPhraseWheel(gmPhraseWheel.cPhraseWheel):
 	"""Let user select a document comment from all existing comments."""
@@ -199,7 +253,6 @@ u"""select * from ((
 				self.data = gmMedDoc.create_document_type(self.GetValue().strip())['pk_doc_type']	# FIXME: error handling
 		return self.data
 #============================================================
-# slightly misnamed due to 
 class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 	def __init__(self, *args, **kwds):
 		"""Support parts and docs now.
@@ -332,7 +385,7 @@ class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 
 		doc_type = self._PhWheel_doc_type.GetData(can_create = True)
 		if doc_type is None:
-			gmDispatcher.send(signal=gmSignals.statustext(), msg=_('Cannot change document type to [%s].') % self._PhWheel_doc_type.GetValue().strip())
+			gmDispatcher.send(signal='statustext', msg=_('Cannot change document type to [%s].') % self._PhWheel_doc_type.GetValue().strip())
 			return False
 
 		# since the phrasewheel operates on the active
@@ -401,7 +454,6 @@ class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 			self._PRW_doc_comment.set_context(context = 'pk_doc_type', val = pk_doc_type)
 		return True
 #============================================================
-# FIXME: this must listen to patient change signals ...
 class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_PluginMixin):
 	def __init__(self, *args, **kwds):
 		wxgScanIdxPnl.wxgScanIdxPnl.__init__(self, *args, **kwds)
@@ -412,9 +464,11 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 		self.__init_ui_data()
 		self._PhWheel_doc_type.add_callback_on_lose_focus(self._on_doc_type_loses_focus)
 
-		# make me a file drop target
+		# make me and listctrl a file drop target
 		dt = gmGuiHelpers.cFileDropTarget(self)
 		self.SetDropTarget(dt)
+		dt = gmGuiHelpers.cFileDropTarget(self._LBOX_doc_pages)
+		self._LBOX_doc_pages.SetDropTarget(dt)
 
 		# do not import globally since we might want to use
 		# this module without requiring any scanner to be available
@@ -426,7 +480,7 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 	def add_filenames(self, filenames):
 		pat = gmPerson.gmCurrentPatient()
 		if not pat.is_connected():
-			gmDispatcher.send(signal=gmSignals.statustext(), msg=_('Cannot accept new documents. No active patient.'))
+			gmDispatcher.send(signal='statustext', msg=_('Cannot accept new documents. No active patient.'))
 			return
 
 		# dive into folders dropped onto us and extract files (one level deep only)
@@ -434,7 +488,7 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 		for pathname in filenames:
 			try:
 				files = os.listdir(pathname)
-				gmDispatcher.send(signal=gmSignals.statustext(), msg=_('Extracting files from folder [%s] ...') % pathname)
+				gmDispatcher.send(signal='statustext', msg=_('Extracting files from folder [%s] ...') % pathname)
 				for file in files:
 					fullname = os.path.join(pathname, file)
 					if not os.path.isfile(fullname):
@@ -544,18 +598,20 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 			dbcfg = gmCfg.cCfgSQL()
 			device = dbcfg.get2 (
 				option =  'external.xsane.default_device',
-				workplace = gmPerson.gmCurrentProvider().workplace,
-				bias = 'workplace'
+				workplace = gmSurgery.gmCurrentPractice().active_workplace,
+				bias = 'workplace',
+				default = ''
 			)
+			if device.strip() == u'':
+				device = None
 			if device is not None:
 				return device
 
-		devices = self.scan_module.get_devices()
-		if devices is False:
-			gmGuiHelpers.gm_statustext (
-				_('There is no scanner support installed on this machine.'),
-				gmLog.lWarn
-			)
+		try:
+			devices = self.scan_module.get_devices()
+		except:
+			_log.LogException('cannot retrieve list of image sources')
+			gmDispatcher.send(signal = 'statustext', msg = _('There is no scanner support installed on this machine.'))
 			return None
 
 		if devices is None:
@@ -564,26 +620,27 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 			return None
 
 		if len(devices) == 0:
-			gmGuiHelpers.gm_statustext (
-				_('Cannot find an active scanner.'),
-				gmLog.lWarn
-			)
+			gmDispatcher.send(signal = 'statustext', msg = _('Cannot find an active scanner.'))
 			return None
 
-		device_names = []
-		for device in devices:
-			device_names.append('%s (%s)' % (device[2], device[0]))
-		# wxPython does not support client data in wxSingleChoiceDialog
-		device_idx = gmGuiHelpers.gm_SingleChoiceDialog (
-			aMessage = _('Select an image capture device'),
-			aTitle = _('device selection'),
-			choices = device_names
+#		device_names = []
+#		for device in devices:
+#			device_names.append('%s (%s)' % (device[2], device[0]))
+
+		device = gmListWidgets.get_choices_from_list (
+			parent = self,
+			msg = _('Select an image capture device'),
+			caption = _('device selection'),
+			choices = [ '%s (%s)' % (d[2], d[0]) for d in devices ],
+			columns = [_('Device')],
+			data = devices,
+			single_selection = True
 		)
-		if device_idx is False:
+		if device is None:
 			return None
 
 		# FIXME: add support for actually reconfiguring
-		return devices[device_idx][0]
+		return device[0]
 	#--------------------------------------------------------
 	# event handling API
 	#--------------------------------------------------------
@@ -591,7 +648,7 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 
 		chosen_device = self.get_device_to_use()
 
-		tmpdir = os.path.expanduser(os.path.join('~', 'gnumed', 'tmp'))
+		tmpdir = os.path.expanduser(os.path.join('~', '.gnumed', 'tmp'))
 		try:
 			gmTools.mkdir(tmpdir)
 		except:
@@ -599,16 +656,22 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 
 		# FIXME: configure whether to use XSane or sane directly
 		# FIXME: add support for xsane_device_settings argument
-		fnames = self.scan_module.acquire_pages_into_files (
-			device = chosen_device,
-			delay = 5,
-			tmpdir = tmpdir,
-			calling_window = self
-		)
-
-		if fnames is False:
+		try:
+			fnames = self.scan_module.acquire_pages_into_files (
+				device = chosen_device,
+				delay = 5,
+				tmpdir = tmpdir,
+				calling_window = self
+			)
+		except ImportError:
 			gmGuiHelpers.gm_show_error (
-				aMessage = _('Page could not be acquired from source.'),
+				aMessage = _(
+					'No pages could be acquired from the source.\n\n'
+					'This may mean the scanner driver is not properly installed\n\n'
+					'On Windows you must install the TWAIN Python module\n'
+					'while on Linux and MacOSX it is recommended to install\n'
+					'the XSane package.'
+				),
 				aTitle = _('acquiring page')
 			)
 			return None
@@ -794,7 +857,7 @@ from your computer.""") % page_fname,
 		show_id = bool (
 			cfg.get2 (
 				option = 'horstspace.scan_index.show_doc_id',
-				workplace = gmPerson.gmCurrentProvider().workplace,
+				workplace = gmSurgery.gmCurrentPractice().active_workplace,
 				bias = 'user'
 			)
 		)
@@ -817,7 +880,7 @@ off this message in the GNUmed configuration.""") % ref
 
 		# prepare for next document
 		self.__init_ui_data()
-		gmDispatcher.send(signal=gmSignals.statustext(), msg=_('Successfully saved new document.'))
+		gmDispatcher.send(signal='statustext', msg=_('Successfully saved new document.'))
 		return True
 	#--------------------------------------------------------
 	def _startover_btn_pressed(self, evt):
@@ -835,54 +898,36 @@ off this message in the GNUmed configuration.""") % ref
 			self._PRW_doc_comment.set_context(context = 'pk_doc_type', val = pk_doc_type)
 		return True
 #============================================================
-class cSelectablySortedDocTreePnl(wxgSelectablySortedDocTreePnl.wxgSelectablySortedDocTreePnl, gmRegetMixin.cRegetOnPaintMixin):
-	"""A document tree that can be sorted."""
-	def __init__(self, *args, **kwds):
-		wxgSelectablySortedDocTreePnl.wxgSelectablySortedDocTreePnl.__init__(self, *args, **kwds)
-		gmRegetMixin.cRegetOnPaintMixin.__init__(self)
-		self.__register_interests()
-	#-------------------------------------------------------
-	# reget mixin API
-	#--------------------------------------------------------
-	def __register_interests(self):
-		gmDispatcher.connect(signal=gmSignals.post_patient_selection(), receiver=self._schedule_data_reget)
-	#-------------------------------------------------------
-	def _populate_with_data(self):
-		"""Fills UI with data."""
-		if not self._doc_tree.refresh():
-			_log.Log(gmLog.lErr, "cannot update document tree")
-			return False
-		self._doc_tree.SelectItem(self._doc_tree.root)
-		return True
+class cSelectablySortedDocTreePnl(wxgSelectablySortedDocTreePnl.wxgSelectablySortedDocTreePnl):
+	"""A panel with a document tree which can be sorted."""
 	#--------------------------------------------------------
 	# inherited event handlers
 	#--------------------------------------------------------
 	def _on_sort_by_age_selected(self, evt):
-		self._doc_tree.set_sort_mode(mode = 'age')
-		self._doc_tree.refresh()
+		self._doc_tree.sort_mode = 'age'
 		self._doc_tree.SetFocus()
 		self._rbtn_sort_by_age.SetValue(True)
 	#--------------------------------------------------------
 	def _on_sort_by_review_selected(self, evt):
-		self._doc_tree.set_sort_mode(mode = 'review')
-		self._doc_tree.refresh()
+		self._doc_tree.sort_mode = 'review'
 		self._doc_tree.SetFocus()
 		self._rbtn_sort_by_review.SetValue(True)
 	#--------------------------------------------------------
 	def _on_sort_by_episode_selected(self, evt):
-		self._doc_tree.set_sort_mode(mode = 'episode')
-		self._doc_tree.refresh()
+		self._doc_tree.sort_mode = 'episode'
 		self._doc_tree.SetFocus()
 		self._rbtn_sort_by_episode.SetValue(True)
 	#--------------------------------------------------------
 	def _on_sort_by_type_selected(self, evt):
-		self._doc_tree.set_sort_mode(mode = 'type')
-		self._doc_tree.refresh()
+		self._doc_tree.sort_mode = 'type'
 		self._doc_tree.SetFocus()
 		self._rbtn_sort_by_type.SetValue(True)
 #============================================================
-class cDocTree(wx.TreeCtrl):
+class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
+	# FIXME: handle expansion state
 	"""This wx.TreeCtrl derivative displays a tree view of stored medical documents.
+
+	It listens to document and patient changes and updated itself accordingly.
 	"""
 	_sort_modes = ['age', 'review', 'episode', 'type']
 	_root_node_labels = None
@@ -893,6 +938,8 @@ class cDocTree(wx.TreeCtrl):
 		kwds['style'] = wx.TR_NO_BUTTONS | wx.NO_BORDER
 		wx.TreeCtrl.__init__(self, parent, id, *args, **kwds)
 
+		gmRegetMixin.cRegetOnPaintMixin.__init__(self)
+
 		tmp = _('available documents (%s)')
 		cDocTree._root_node_labels = {
 			'age': tmp % _('most recent on top'),
@@ -902,20 +949,55 @@ class cDocTree(wx.TreeCtrl):
 		}
 
 		self.root = None
-		self.__sort_mode = None
-		self.set_sort_mode()
-		self.__pat = gmPerson.gmCurrentPatient()
+		self.__sort_mode = 'age'
 
-		self.__register_events()
+		self.__register_interests()
+		self._schedule_data_reget()
 	#--------------------------------------------------------
 	# external API
 	#--------------------------------------------------------
-	def refresh(self):
-		if not self.__pat.is_connected():
-			gmGuiHelpers.gm_statustext (
-				_('Cannot load documents. No active patient.'),
-				gmLog.lErr
-			)
+	def display_selected_part(self, *args, **kwargs):
+
+		node = self.GetSelection()
+		node_data = self.GetPyData(node)
+
+		if not isinstance(node_data, gmMedDoc.cMedDocPart):
+			return True
+
+		self.__display_part(part = node_data)
+		return True
+	#--------------------------------------------------------
+	# properties
+	#--------------------------------------------------------
+	def _get_sort_mode(self):
+		return self.__sort_mode
+	#-----
+	def _set_sort_mode(self, mode):
+		if mode is None:
+			mode = 'age'
+
+		if mode == self.__sort_mode:
+			return
+
+		if mode not in cDocTree._sort_modes:
+			raise ValueError('invalid document tree sort mode [%s], valid modes: %s' % (mode, cDocTree._sort_modes))
+
+		self.__sort_mode = mode
+
+		curr_pat = gmPerson.gmCurrentPatient()
+		if not curr_pat.is_connected():
+			return
+
+		self._schedule_data_reget()
+	#-----
+	sort_mode = property(_get_sort_mode, _set_sort_mode)
+	#--------------------------------------------------------
+	# reget-on-paint API
+	#--------------------------------------------------------
+	def _populate_with_data(self):
+		curr_pat = gmPerson.gmCurrentPatient()
+		if not curr_pat.is_connected():
+			gmDispatcher.send(signal = 'statustext', msg = _('Cannot load documents. No active patient.'))
 			return False
 
 		if not self.__populate_tree():
@@ -923,21 +1005,19 @@ class cDocTree(wx.TreeCtrl):
 
 		return True
 	#--------------------------------------------------------
-	def set_sort_mode(self, mode='age'):
-		if mode not in cDocTree._sort_modes:
-			_log.Log(gmLog.Err, 'invalid document tree sort mode [%s], valid modes: %s' % (mode, cDocTree._sort_modes))
-		if self.__sort_mode == mode:
-			return True
-		self.__sort_mode = mode
-	#--------------------------------------------------------
 	# internal helpers
 	#--------------------------------------------------------
-	def __register_events(self):
+	def __register_interests(self):
 		# connect handlers
 		wx.EVT_TREE_ITEM_ACTIVATED (self, self.GetId(), self._on_activate)
 		wx.EVT_TREE_ITEM_RIGHT_CLICK (self, self.GetId(), self.__on_right_click)
 
 #		 wx.EVT_LEFT_DCLICK(self.tree, self.OnLeftDClick)
+
+		gmDispatcher.connect(signal = u'pre_patient_selection', receiver = self._on_pre_patient_selection)
+		gmDispatcher.connect(signal = u'post_patient_selection', receiver = self._on_post_patient_selection)
+		gmDispatcher.connect(signal = u'doc_mod_db', receiver = self._on_doc_mod_db)
+		gmDispatcher.connect(signal = u'doc_page_mod_db', receiver = self._on_doc_page_mod_db)
 	#--------------------------------------------------------
 	def __populate_tree(self):
 
@@ -952,12 +1032,13 @@ class cDocTree(wx.TreeCtrl):
 		self.SetItemHasChildren(self.root, False)
 
 		# read documents from database
-		docs_folder = self.__pat.get_document_folder()
+		curr_pat = gmPerson.gmCurrentPatient()
+		docs_folder = curr_pat.get_document_folder()
 		docs = docs_folder.get_documents()
 		if docs is None:
-			name = self.__pat.get_names()
+			name = curr_pat.get_names()
 			gmGuiHelpers.gm_show_error (
-				aMessage = _('Error searching documents for patient\n[%s %s].') % (name['first'], name['last']),
+				aMessage = _('Error searching documents for patient\n[%s %s].') % (name['firstnames'], name['lastnames']),
 				aTitle = _('loading document list')
 			)
 			# avoid recursion of GUI updating
@@ -974,18 +1055,12 @@ class cDocTree(wx.TreeCtrl):
 		# add our documents as first level nodes
 		intermediate_nodes = {}
 		for doc in docs:
-			if doc['comment'] is not None:
-				cmt = '%s' % doc['comment']
-			else:
-				cmt = _('no comment available')
 
 			parts = doc.get_parts()
-			page_num = len(parts)
 
-			if doc['ext_ref'] is not None:
-				ref = '>%s<' % doc['ext_ref']
-			else:
-				ref = _('no reference ID found')
+			cmt = gmTools.coalesce(doc['comment'], _('no comment available'))
+			page_num = len(parts)
+			ref = gmTools.coalesce(initial = doc['ext_ref'], instead = _('no reference ID found'), template_initial = u'>%s<')
 
 			if doc.has_unreviewed_parts():
 				review = '!'
@@ -1025,18 +1100,13 @@ class cDocTree(wx.TreeCtrl):
 			for part in parts:
 
 				pg = _('page %2s') % part['seq_idx']
-
 				cmt = gmTools.coalesce(part['obj_comment'], _("no comment available"))
-
-				if part['size'] == 0:
-					sz = _('0 bytes - data missing ?')
-				else:
-					sz = _('%s bytes') % part['size']
-
-				if part['reviewed'] or part['reviewed_by_you'] or part['reviewed_by_intended_reviewer']:
-					rev = ''
-				else:
-					rev = ' [%s]' % _('unreviewed')
+				sz = gmTools.size2str(part['size'])
+				rev = gmTools.bool2str (
+					bool = part['reviewed'] or part['reviewed_by_you'] or part['reviewed_by_intended_reviewer'],
+					true_str = u'',
+					false_str = ' [%s]' % _('unreviewed')
+				)
 
 #				if part['clinically_relevant']:
 #					rel = ' [%s]' % _('Cave')
@@ -1051,7 +1121,8 @@ class cDocTree(wx.TreeCtrl):
 		self.SortChildren(self.root)
 		self.SelectItem(self.root)
 
-		# and uncollapse
+		# FIXME: apply expansion state if available or else ...
+		# FIXME: ... uncollapse to default state
 		self.Expand(self.root)
 		if self.__sort_mode in ['episode', 'type']:
 			for key in intermediate_nodes.keys():
@@ -1148,7 +1219,27 @@ class cDocTree(wx.TreeCtrl):
 	#------------------------------------------------------------------------
 	# event handlers
 	#------------------------------------------------------------------------
-	def _on_activate (self, event):
+	def _on_doc_mod_db(self, *args, **kwargs):
+		# FIXME: remember current expansion state
+		wx.CallAfter(self._schedule_data_reget)
+	#------------------------------------------------------------------------
+	def _on_doc_page_mod_db(self, *args, **kwargs):
+		# FIXME: remember current expansion state
+		wx.CallAfter(self._schedule_data_reget)
+	#------------------------------------------------------------------------
+	def _on_pre_patient_selection(self, *args, **kwargs):
+		# FIXME: self.__store_expansion_history_in_db
+
+		# empty out tree
+		if self.root is not None:
+			self.DeleteAllItems()
+		self.root = None
+	#------------------------------------------------------------------------
+	def _on_post_patient_selection(self, *args, **kwargs):
+		# FIXME: self.__load_expansion_history_from_db (but not apply it !)
+		self._schedule_data_reget()
+	#------------------------------------------------------------------------
+	def _on_activate(self, event):
 		node = event.GetItem()
 		node_data = self.GetPyData(node)
 
@@ -1170,6 +1261,7 @@ class cDocTree(wx.TreeCtrl):
 		return True
 	#--------------------------------------------------------
 	def __on_right_click(self, evt):
+
 		node = evt.GetItem()
 		self.__curr_node_data = self.GetPyData(node)
 
@@ -1216,6 +1308,16 @@ class cDocTree(wx.TreeCtrl):
 		menu.AppendItem(wx.MenuItem(menu, ID, _('Export to disk')))
 		wx.EVT_MENU(menu, ID, self.__export_doc_to_disk)
 
+		# edit encounter
+		ID = wx.NewId()
+		menu.AppendItem(wx.MenuItem(menu, ID, _('Edit corresponding consultation')))
+		wx.EVT_MENU(menu, ID, self.__edit_consultation_details)
+
+		# delete for good
+		ID = wx.NewId()
+		menu.AppendItem(wx.MenuItem(menu, ID, _('Delete')))
+		wx.EVT_MENU(menu, ID, self.__delete_document)
+
 		# show descriptions
 		descriptions = self.__curr_node_data.get_descriptions()
 		desc_menu = wx.Menu()
@@ -1268,77 +1370,66 @@ class cDocTree(wx.TreeCtrl):
 			)
 			return None
 
-		# get export directory for temporary files
-		def_tmp_dir = os.path.expanduser(os.path.join('~', 'gnumed', 'tmp'))
-		if not os.path.isdir(def_tmp_dir):
-			try:
-				os.makedirs(def_tmp_dir)
-			except:
-				def_tmp_dir = None
 		cfg = gmCfg.cCfgSQL()
-		tmp_dir = cfg.get2 (
-			option = "horstspace.tmp_dir",
-			workplace = gmPerson.gmCurrentProvider().workplace,
-			bias = 'workplace',
-			default = def_tmp_dir
+
+		# get export directory for temporary files
+		tmp_dir = gmTools.coalesce (
+			cfg.get2 (
+				option = "horstspace.tmp_dir",
+				workplace = gmSurgery.gmCurrentPractice().active_workplace,
+				bias = 'workplace'
+			),
+			os.path.expanduser(os.path.join('~', '.gnumed', 'tmp', 'docs'))
 		)
-		exp_base = os.path.abspath(os.path.expanduser(os.path.join(tmp_dir, 'docs')))
-		if not os.path.isdir(exp_base):
-			_log.Log(gmLog.lWarn, "The directory [%s] does not exist ! Falling back to default temporary directory." % exp_base) # which is None == tempfile.tempdir == use system defaults
-			exp_base = None
-		else:
-			_log.Log(gmLog.lData, "working into directory [%s]" % exp_base)
+		_log.Log(gmLog.lData, "working into directory [%s]" % tmp_dir)
 
 		# determine database export chunk size
 		chunksize = int(cfg.get2 (
 			option = "horstspace.blob_export_chunk_size",
-			workplace = gmPerson.gmCurrentProvider().workplace,
+			workplace = gmSurgery.gmCurrentPractice().active_workplace,
 			bias = 'workplace',
 			default = 1 * 1024 * 1024		# 1 MB
 		))
 
-		# retrieve doc part
-		fname = part.export_to_file(aTempDir = exp_base, aChunkSize = chunksize)
-		if fname is None:
-			_log.Log(gmLog.lErr, "cannot export doc part [%s] data from database" % part['pk_obj'])
+		# shall we force blocking during view ?
+		block_during_view = bool( cfg.get2 (
+			option = 'horstspace.document_viewer.block_during_view',
+			workplace = gmSurgery.gmCurrentPractice().active_workplace,
+			bias = 'user',
+			default = None
+		))
+
+		# display it
+		successful, msg = part.display_via_mime (
+			tmpdir = tmp_dir,
+			chunksize = chunksize,
+			block = block_during_view
+		)
+		if not successful:
 			gmGuiHelpers.gm_show_error (
-				aMessage = _('Cannot export document part from database to file.'),
+				aMessage = _('Cannot display document part:\n%s') % msg,
 				aTitle = _('showing document')
 			)
 			return None
 
-		# display it
-		block_during_view = bool( cfg.get2 (
-			option = 'horstspace.document_viewer.block_during_view',
-			workplace = gmPerson.gmCurrentProvider().workplace,
-			bias = 'user',
-			default = None
-		))
-		(result, msg) = gmMimeLib.call_viewer_on_file(fname, block=block_during_view)
-		if not result:
-			gmGuiHelpers.gm_show_error (
-				aMessage = _('Cannot display document part:\n%s') % msg,
-				aTitle = _('displaying page')
-			)
-			return None
-
 		# handle review after display
-		review_after_display = cfg.get2 (
+		# 0: never
+		# 1: always
+		# 2: if no review by myself exists yet
+		review_after_display = int(cfg.get2 (
 			option = 'horstspace.document_viewer.review_after_display',
-			workplace = gmPerson.gmCurrentProvider().workplace,
+			workplace = gmSurgery.gmCurrentPractice().active_workplace,
 			bias = 'user',
 			default = 2
-		)
-		# always review
-		if review_after_display == 1:
+		))
+		if review_after_display == 1:			# always review
 			self.__review_part(part=part)
-		# review if no review by me exists
-		elif review_after_display == 2:
+		elif review_after_display == 2:			# review if no review by me exists
 			review_by_me = filter(lambda rev: rev['is_review_by_you'], part.get_reviews())
 			if len(review_by_me) == 0:
 				self.__review_part(part=part)
 
-		return 1
+		return True
 	#--------------------------------------------------------
 	def __review_part(self, part=None):
 		dlg = cReviewDocPartDlg (
@@ -1346,11 +1437,15 @@ class cDocTree(wx.TreeCtrl):
 			id = -1,
 			part = part
 		)
-		if dlg.ShowModal() == wx.ID_OK:
-			self.__populate_tree()
+		dlg.ShowModal()
 	#--------------------------------------------------------
 	# document level context menu handlers
 	#--------------------------------------------------------
+	def __edit_consultation_details(self, evt):
+		enc = gmEMRStructItems.cEncounter(aPK_obj=self.__curr_node_data['pk_encounter'])
+		dlg = gmEMRStructWidgets.cEncounterEditAreaDlg(parent=self, encounter=enc)
+		dlg.ShowModal()
+	#--------------------------------------------------------
 	def __export_doc_to_disk(self, evt):
 		"""Export document into directory.
 
@@ -1365,7 +1460,7 @@ class cDocTree(wx.TreeCtrl):
 		)
 		def_dir = os.path.expanduser(os.path.join('~', 'gnumed', 'export', 'docs', pat['dirname'], dname))
 		gmTools.mkdir(def_dir)
-		
+
 		dlg = wx.DirDialog (
 			parent = self,
 			message = _('Save document into directory ...'),
@@ -1383,20 +1478,121 @@ class cDocTree(wx.TreeCtrl):
 		fnames = self.__curr_node_data.export_parts_to_files(export_dir=dirname)
 		wx.EndBusyCursor()
 
-		gmDispatcher.send(signal=gmSignals.statustext(), msg=_('Successfully exported %s pages into the directory [%s].') % (len(fnames), dirname))
+		gmDispatcher.send(signal='statustext', msg=_('Successfully exported %s pages into the directory [%s].') % (len(fnames), dirname))
 
 		return True
+	#--------------------------------------------------------
+	def __delete_document(self, evt):
+		curr_pat = gmPerson.gmCurrentPatient()
+		emr = curr_pat.get_emr()
+		enc = emr.get_active_encounter()
+		gmMedDoc.delete_document(document_id = self.__curr_node_data['pk_doc'], encounter_id = enc['pk_encounter'])
 #============================================================
 # main
 #------------------------------------------------------------
 if __name__ == '__main__':
 	_log.SetAllLogLevels(gmLog.lData)
+	gmI18N.activate_locale()
+	gmI18N.install_domain(domain = 'gnumed')
 
-	print "==> the syntax seems OK"
-	print "==> please write a real unit test"
+	#----------------------------------------
+	#----------------------------------------
+	if (len(sys.argv) > 1) and (sys.argv[1] == 'test'):
+#		test_*()
+		pass
 
 #============================================================
 # $Log: gmMedDocWidgets.py,v $
+# Revision 1.150.2.1  2008/01/08 22:39:31  ncq
+# - 'first' / 'last' -> 'firstnames' / 'lastnames'
+#
+# Revision 1.150  2007/11/05 11:41:46  ncq
+# - use blobs.delete_document()
+#
+# Revision 1.149  2007/10/31 22:07:18  ncq
+# - delete document from context menu
+#
+# Revision 1.148  2007/10/31 11:26:18  ncq
+# - hide less exceptions
+#
+# Revision 1.147  2007/10/29 13:22:32  ncq
+# - make cDocTree a lot more self contained:
+#   - make it a reget mixin child
+#   - make sort_mode a property scheduling reload on set
+#   - listen to patient changes
+#     - empty tree on pre_sel
+# 	- schedule reload on post_sel
+#   - listen to doc and page changes and schedule appropriate reloads
+#
+# Revision 1.146  2007/10/12 07:27:02  ncq
+# - check in drop target fix
+#
+# Revision 1.145  2007/10/07 12:32:41  ncq
+# - workplace property now on gmSurgery.gmCurrentPractice() borg
+#
+# Revision 1.144  2007/09/07 10:57:54  ncq
+# - document review_after_display
+#
+# Revision 1.143  2007/08/29 14:43:06  ncq
+# - factor out forms/letters related code
+# - fix syntax error re stray, gmLog.L* consts
+#
+# Revision 1.142  2007/08/28 14:18:13  ncq
+# - no more gm_statustext()
+#
+# Revision 1.141  2007/08/20 22:12:49  ncq
+# - support _on_load_button_pressed in form template editor
+#
+# Revision 1.140  2007/08/20 16:23:52  ncq
+# - support editing form templates from create_new_letter
+# - cFormTemplateEditAreaDlg
+#
+# Revision 1.139  2007/08/20 14:29:31  ncq
+# - cleanup, start of test suite
+# - form template edit area
+#
+# Revision 1.138  2007/08/15 09:20:43  ncq
+# - use cOOoLetter.show()
+# - cleanup
+# - use gmTools.size2str()
+#
+# Revision 1.137  2007/08/13 22:11:38  ncq
+# - use cFormTemplate
+# - pass placeholder handler to form instance handler
+#
+# Revision 1.136  2007/08/12 00:10:55  ncq
+# - improve create_new_letter()
+# - (_)save_file_as_new_document() and listen for 'import_document_from_file'
+# - no more gmSignals.py
+#
+# Revision 1.135  2007/08/09 07:59:42  ncq
+# - streamline __display_part() with part.display_via_mime()
+#
+# Revision 1.134  2007/07/22 10:04:23  ncq
+# - streamline create_new_letter()
+#
+# Revision 1.133  2007/07/22 09:27:28  ncq
+# - create_new_letter()
+# - adjust to get_choice_from_list() changes
+# - tmp/ now in .gnumed/
+#
+# Revision 1.132  2007/07/11 21:10:31  ncq
+# - cleanup
+#
+# Revision 1.131  2007/06/28 12:39:37  ncq
+# - make pages listbox in scan/index panel be a drop target itself, too
+# - handle preset device = '' as unconfigured
+#
+# Revision 1.130  2007/06/18 20:38:32  ncq
+# - use gmListWidgets.get_choice_from_list()
+#
+# Revision 1.129  2007/06/12 13:24:48  ncq
+# - allow editing of encounter corresponding to a document
+#
+# Revision 1.128  2007/06/10 10:17:36  ncq
+# - gmScanBackend now uses exceptions for error handling
+# - improved error message when no sanner driver found
+#
 # Revision 1.127  2007/05/21 14:49:20  ncq
 # - use pat['dirname'] for export
 #
