@@ -10,19 +10,22 @@ generator.
 """
 #============================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gmPatSearchWidgets.py,v $
-# $Id: gmPatSearchWidgets.py,v 1.80.2.1 2007/06/01 15:15:47 ncq Exp $
-__version__ = "$Revision: 1.80.2.1 $"
+# $Id: gmPatSearchWidgets.py,v 1.97.2.1 2008/01/08 22:39:31 ncq Exp $
+__version__ = "$Revision: 1.97.2.1 $"
 __author__ = "K.Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = 'GPL (for details see http://www.gnu.org/)'
 
-import sys, os.path, time, glob, locale, datetime as pyDT, webbrowser, fileinput
+import sys, os.path, time, glob, datetime as pyDT, re as regex
+
 
 import wx
 
-from Gnumed.pycommon import gmLog, gmDispatcher, gmSignals, gmPG2, gmI18N, gmCfg, gmTools, gmDateTime, gmMatchProvider, gmMimeLib
-from Gnumed.business import gmPerson, gmKVK, gmDataMining
+
+from Gnumed.pycommon import gmLog, gmDispatcher, gmSignals, gmPG2, gmI18N, gmCfg, gmTools, gmDateTime, gmMatchProvider
+from Gnumed.business import gmPerson, gmKVK, gmSurgery
 from Gnumed.wxpython import gmGuiHelpers, gmDemographicsWidgets
-from Gnumed.wxGladeWidgets import wxgSelectPersonFromListDlg, wxgSelectPersonDTOFromListDlg, wxgDataMiningPnl
+from Gnumed.wxGladeWidgets import wxgSelectPersonFromListDlg, wxgSelectPersonDTOFromListDlg
+
 
 _log = gmLog.gmDefLog
 _log.Log(gmLog.lInfo, __version__)
@@ -32,184 +35,6 @@ _cfg = gmCfg.gmDefCfgFile
 ID_PatPickList = wx.NewId()
 ID_BTN_AddNew = wx.NewId()
 
-#============================================================
-# FIXME: this class is perhaps a bit oddly placed here
-class cDataMiningPnl(wxgDataMiningPnl.wxgDataMiningPnl):
-
-	def __init__(self, *args, **kwargs):
-		wxgDataMiningPnl.wxgDataMiningPnl.__init__(self, *args, **kwargs)
-
-		self.__init_ui()
-
-		# make me a file drop target
-		dt = gmGuiHelpers.cFileDropTarget(self)
-		self.SetDropTarget(dt)
-	#--------------------------------------------------------
-	def __init_ui(self):
-		mp = gmMatchProvider.cMatchProvider_SQL2 (
-			queries = [u'select distinct on (label) cmd, label from cfg.report_query where label %(fragment_condition)s or cmd %(fragment_condition)s']
-		)
-		mp.setThresholds(2,3,5)
-		self._PRW_report_name.matcher = mp
-		self._PRW_report_name.add_callback_on_selection(callback = self._on_report_selected)
-	#--------------------------------------------------------
-	def _on_report_selected(self, *args, **kwargs):
-		self._TCTRL_query.SetValue(self._PRW_report_name.GetData())
-		self._BTN_run.SetFocus()
-	#--------------------------------------------------------
-	# file drop target API
-	#--------------------------------------------------------
-	def add_filenames(self, filenames):
-		# act on first file only
-		fname = filenames[0]
-		# act on text files only
-		mime_type = gmMimeLib.guess_mimetye(fname)
-		if not mime_type.startswith('text/'):
-			gmDispatcher.send(signal=gmSignals.statustext(), msg = _('Cannot read SQL from [%s]. Not a text file.') % fname, beep = True)
-			return False
-		# act on "small" files only
-		stat_val = os.stat(fname)
-		if stat_val.st_size > 2000:
-			gmDispatcher.send(signal=gmSignals.statustext(), msg = _('Cannot read SQL from [%s]. File too big (> 2000 bytes).') % fname, beep = True)
-			return False
-		# all checks passed
-		for line in fileinput.input(fname):
-			self._TCTRL_query.AppendText(line)
-	#--------------------------------------------------------
-	# notebook plugin API
-	#--------------------------------------------------------
-	def repopulate_ui(self):
-		pass
-	#--------------------------------------------------------
-	# event handlers
-	#--------------------------------------------------------
-	def _on_list_item_activated(self, evt):
-		data = self._LCTRL_result.get_selected_item_data()
-		try:
-			pk_pat = data['pk_patient']
-		except KeyError:
-			gmGuiHelpers.gm_show_warning (
-				_(
-				'Cannot activate patient.\n\n'
-				'The report result list does not contain\n'
-				'a column named "pk_patient".\n\n'
-				'You may want to use the SQL "AS" column alias\n'
-				'syntax to make your query return such a column.\n'
-				),
-				_('activating patient from report result')
-			)
-			return
-		pat = gmPerson.cPatient(aPK_obj = pk_pat)
-		gmPerson.set_active_patient(patient = pat)
-	#--------------------------------------------------------
-	def _on_contribute_button_pressed(self, evt):
-		report = self._PRW_report_name.GetValue().strip()
-		if report == u'':
-			return
-		query = self._TCTRL_query.GetValue().strip()
-		if query == u'':
-			return
-
-		auth = {'user': gmTools.default_mail_sender, 'password': u'gm/bugs/gmx'}
-		msg = u"""
-To: gnumed-devel@gnu.org
-From: GNUmed Report Generator <gnumed@gmx.net>
-Subject: user contributed report
-
-This is a report definition contributed
-by a GNUmed user:
-
-#--------------------------------------
-
-%s
-
-%s
-
-#--------------------------------------
-
-The GNUmed client.
-""" % (report, query)
-
-		if not gmTools.send_mail(message = msg, auth = auth):
-			gmDispatcher.send(signal = gmSignals.statustext(), msg = _('Unable to send mail. Cannot contribute report [%s] to GNUmed community.') % report, beep = True)
-			return False
-
-		gmDispatcher.send(signal = gmSignals.statustext(), msg = _('Thank you for your contribution to the GNUmed community !'), beep = False)
-		return True
-	#--------------------------------------------------------
-	def _on_schema_button_pressed(self, evt):
-		webbrowser.open(u'http://wiki.gnumed.de/bin/view/Gnumed/DatabaseSchema', new=0, autoraise=1)
-	#--------------------------------------------------------
-	def _on_delete_button_pressed(self, evt):
-		report = self._PRW_report_name.GetValue().strip()
-		if report == u'':
-			return True
-		if gmDataMining.delete_report_definition(name=report):
-			self._PRW_report_name.SetText()
-			self._TCTRL_query.SetValue(u'')
-			gmDispatcher.send(signal=gmSignals.statustext(), msg = _('Deleted report definition [%s].') % report, beep=False)
-			return True
-		gmDispatcher.send(signal=gmSignals.statustext(), msg = _('Error deleting report definition [%s].') % report, beep=True)
-		return False
-	#--------------------------------------------------------
-	def _on_clear_button_pressed(self, evt):
-		self._PRW_report_name.SetText()
-		self._TCTRL_query.SetValue(u'')
-	#--------------------------------------------------------
-	def _on_save_button_pressed(self, evt):
-		report = self._PRW_report_name.GetValue().strip()
-		if report == u'':
-			gmDispatcher.send(signal=gmSignals.statustext(), msg = _('Cannot save report definition without name.'), beep=True)
-			return False
-		query = self._TCTRL_query.GetValue().strip()
-		if query == u'':
-			gmDispatcher.send(signal=gmSignals.statustext(), msg = _('Cannot save report definition without query.'), beep=True)
-			return False
-		# FIXME: check for exists and ask for permission
-		if gmDataMining.save_report_definition(name=report, query=query, overwrite=True):
-			gmDispatcher.send(signal=gmSignals.statustext(), msg = _('Saved report definition [%s].') % report, beep=False)
-			return True
-		gmDispatcher.send(signal=gmSignals.statustext(), msg = _('Error saving report definition [%s].') % report, beep=True)
-		return False
-	#--------------------------------------------------------
-	def _on_run_button_pressed(self, evt):
-		query = self._TCTRL_query.GetValue().strip().strip(';')
-		if query == u'':
-			return True
-
-		self._LCTRL_result.set_columns()
-
-		# FIXME: make configurable
-		query = u'select * from (' + query + u') as real_query limit 1024'
-		try:
-			# read-only only for safety reasons
-			rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': query}], get_col_idx = True)
-		except:
-			self._LCTRL_result.set_columns([_('Error')])
-			self._LCTRL_result.InsertStringItem(sys.maxint, label = _('The query failed.'))
-			gmDispatcher.send(gmSignals.statustext(), msg = _('The query failed.'), beep = True)
-			_log.LogException('report query failed')
-			return False
-
-		if len(rows) == 0:
-			self._LCTRL_result.set_columns([_('Results')])
-			self._LCTRL_result.InsertStringItem(sys.maxint, label = _('Report returned no data.'))
-			gmDispatcher.send(gmSignals.statustext(), msg = _('No data returned for this report.'), beep = True)
-			return True
-
-		cols = [(value, key) for key, value in idx.items()]
-		cols.sort()
-		cols = [pair[1] for pair in cols]
-
-		self._LCTRL_result.set_columns(cols)
-		for row in rows:
-			label = unicode(gmTools.coalesce(row[0], u''))
-			row_num = self._LCTRL_result.InsertStringItem(sys.maxint, label = label)
-			for col_idx in range(1, len(row)):
-				self._LCTRL_result.SetStringItem(index = row_num, col = col_idx, label = unicode(gmTools.coalesce(row[col_idx], u'')))
-
-		self._LCTRL_result.set_column_widths()
-		self._LCTRL_result.set_data(data = rows)
 #============================================================
 class cSelectPersonFromListDlg(wxgSelectPersonFromListDlg.wxgSelectPersonFromListDlg):
 
@@ -252,8 +77,10 @@ class cSelectPersonFromListDlg(wxgSelectPersonFromListDlg.wxgSelectPersonFromLis
 			else:
 				label = u'%s (%s)' % (enc['started'].strftime('%x').decode(gmI18N.get_encoding()), enc['l10n_type'])
 			self._LCTRL_persons.SetStringItem(index = row_num, col = 6, label = label)
-			try: self._LCTRL_persons.SetStringItem(index = row_num, col = 7, label = ident['match_type'])
-			except: pass
+			try: self._LCTRL_persons.SetStringItem(index = row_num, col = 7, label = person['match_type'])
+			except:
+				_log.LogException('cannot set match_type field')
+				self._LCTRL_persons.SetStringItem(index = row_num, col = 7, label = u'??')
 
 		for col in range(len(self.__cols)):
 			self._LCTRL_persons.SetColumnWidth(col=col, width=wx.LIST_AUTOSIZE)
@@ -310,7 +137,10 @@ class cSelectPersonDTOFromListDlg(wxgSelectPersonDTOFromListDlg.wxgSelectPersonD
 			dto = rec['dto']
 			self._LCTRL_persons.SetStringItem(index = row_num, col = 1, label = dto.lastnames)
 			self._LCTRL_persons.SetStringItem(index = row_num, col = 2, label = dto.firstnames)
-			self._LCTRL_persons.SetStringItem(index = row_num, col = 3, label = dto.dob.strftime('%x').decode(gmI18N.get_encoding()))
+			if dto.dob is None:
+				self._LCTRL_persons.SetStringItem(index = row_num, col = 3, label = u'')
+			else:
+				self._LCTRL_persons.SetStringItem(index = row_num, col = 3, label = dto.dob.strftime('%x').decode(gmI18N.get_encoding()))
 			self._LCTRL_persons.SetStringItem(index = row_num, col = 4, label = gmTools.coalesce(dto.gender, ''))
 
 		for col in range(len(self.__cols)):
@@ -368,12 +198,9 @@ def load_persons_from_xdt():
 		if encoding is None:
 			_log.Log(gmLog.lWarn, 'xDT source profile [%s] does not specify an <encoding> for BDT file [%s]' % (profile, name))
 		source = _cfg.get('XDT profile %s' % profile, 'source')
-		if source is None:
-			source = _('unknown')
 		dob_format = _cfg.get('XDT profile %s' % profile, 'DOB format')
 		if dob_format is None:
-			_log.Log(gmLog.lErr, 'XDT profile [%s] does not define a date of birth format in <DOB format>' % profile)
-			continue
+			_log.Log(gmLog.lWarn, 'XDT profile [%s] does not define a date of birth format in <DOB format>' % profile)
 		bdt_files.append({'file': name, 'source': source, 'encoding': encoding, 'dob_format': dob_format})
 
 	dtos = []
@@ -398,31 +225,19 @@ def load_persons_from_xdt():
 			)
 			_log.LogException('cannot access xDT file [%s]' % bdt_file['file'])
 			continue
-
-		except ValueError:
-			gmGuiHelpers.gm_show_info (
-				_(
-				'Cannot load patient from BDT file\n\n'
-				' [%s]'
-				) % bdt_file,
-				_('Activating xDT patient')
-			)
-			_log.LogException('cannot read patient from xDT file [%s]' % bdt_file['file'])
-			continue
-
 		except:
 			gmGuiHelpers.gm_show_error (
 				_(
 				'Cannot load patient from BDT file\n\n'
 				' [%s]'
 				) % bdt_file,
-				_('Activating xDT patient'),
-				aLogLevel = None
+				_('Activating xDT patient')
+			, aLogLevel = None
 			)
 			_log.LogException('cannot read patient from xDT file [%s]' % bdt_file['file'])
 			continue
 
-		dtos.append({'dto': dto, 'source': bdt_file['source']})
+		dtos.append({'dto': dto, 'source': gmTools.coalesce(bdt_file['source'], dto.source)})
 
 	return dtos
 #============================================================
@@ -444,7 +259,11 @@ def load_persons_from_pracsoft_au():
 	fname = _cfg.get('AU PracSoft PATIENTS.IN', 'filename')
 	source = _cfg.get('AU PracSoft PATIENTS.IN', 'source')
 	if fname is not None and source is not None:
-		pracsoft_files.append({'file': os.path.expanduser(fname), 'source': source})
+		fname = os.path.abspath(os.path.expanduser(fname))
+		if os.access(fname, os.R_OK):
+			pracsoft_files.append({'file': os.path.expanduser(fname), 'source': source})
+		else:
+			_log.Log(gmLog.lErr, 'cannot read [%s] in AU PracSoft profile' % fname)
 
 	# and parse them
 	dtos = []
@@ -464,7 +283,7 @@ def load_persons_from_kvks():
 	db_cfg = gmCfg.cCfgSQL()
 	kvk_dir = os.path.abspath(os.path.expanduser(db_cfg.get2 (
 		option = 'DE.KVK.spool_dir',
-		workplace = gmPerson.gmCurrentProvider().workplace,
+		workplace = gmSurgery.gmCurrentPractice().active_workplace,
 		bias = 'workplace',
 		default = u'/var/spool/kvkd/'
 	)))
@@ -474,63 +293,93 @@ def load_persons_from_kvks():
 
 	return dtos
 #============================================================
-def load_patient_from_external_sources(parent=None):
+def load_patient_from_external_sources(parent=None, search_immediately=False):
+	"""Load patient from external source.
 
+	- scan external sources for candidates
+	- let user select source
+	  - if > 1 available: always
+	  - if only 1 available: depending on search_immediately
+	- search for patients matching info from external source
+	- if more than one match:
+	  - let user select patient
+	- if no match:
+	  - create patient
+	- activate patient
+	"""
 	# get DTOs from interfaces
 	dtos = []
 	dtos.extend(load_persons_from_xdt())
 	dtos.extend(load_persons_from_pracsoft_au())
 	dtos.extend(load_persons_from_kvks())
 
+	# no external patients
 	if len(dtos) == 0:
-		gmDispatcher.send(signal=gmSignals.statustext(), msg=_('No patients found in external sources.'))
+		gmDispatcher.send(signal='statustext', msg=_('No patients found in external sources.'))
 		return True
-	elif len(dtos) == 1:
+
+	# one external patient with DOB - already active ?
+	if (len(dtos) == 1) and (dtos[0]['dto'].dob is not None):
+		dto = dtos[0]['dto']
+		# is it already the current patient ?
 		curr_pat = gmPerson.gmCurrentPatient()
 		if curr_pat.is_connected():
-			dto = dtos[0]
 			key_dto = dto.firstnames + dto.lastnames + dto.dob.strftime('%Y-%m-%d') + dto.gender
 			names = curr_pat.get_active_name()
-			key_pat = names['first'] + names['last'] + curr_pat['dob'].strftime('%Y-%m-%d') + curr_pat['gender']
+			key_pat = names['firstnames'] + names['lastnames'] + curr_pat['dob'].strftime('%Y-%m-%d') + curr_pat['gender']
 			_log.Log(gmLog.lData, 'current patient: %s' % key_pat)
 			_log.Log(gmLog.lData, 'dto patient    : %s' % key_dto)
 			if key_dto == key_pat:
-				gmDispatcher.send(signal=gmSignals.statustext(), msg=_('The only external patient is already active in GNUmed.'), beep=False)
+				gmDispatcher.send(signal='statustext', msg=_('The only external patient is already active in GNUmed.'), beep=False)
 				return True
 
-	if parent is None:
-		parent = wx.GetApp().GetTopWindow()
-	dlg = cSelectPersonDTOFromListDlg(parent=parent, id=-1)
-	dlg.set_dtos(dtos=dtos)
-	result = dlg.ShowModal()
-	if result == wx.ID_CANCEL:
-		return True
-	dto = dlg.get_selected_dto()['dto']
-	dlg.Destroy()
+	# one external patient - activate immediately ?
+	if (len(dtos) == 1) and search_immediately:
+		dto = dtos[0]['dto']
 
-	# FIXME: config: delete DTO source after selection, eg KVK
+	# several external patients
+	else:
+		if parent is None:
+			parent = wx.GetApp().GetTopWindow()
+		dlg = cSelectPersonDTOFromListDlg(parent=parent, id=-1)
+		dlg.set_dtos(dtos=dtos)
+		result = dlg.ShowModal()
+		if result == wx.ID_CANCEL:
+			return True
+		dto = dlg.get_selected_dto()['dto']
+		dlg.Destroy()
 
 	# search
-	searcher = gmPerson.cPatientSearcher_SQL()
-	idents = searcher.get_identities(dto = dto)
+#	searcher = gmPerson.cPatientSearcher_SQL()
+#	idents = searcher.get_identities(dto = dto)
+	idents = dto.get_candidate_identities(can_create=True)
 
-	ident = None
-	if len(idents) == 0:
-		ident = gmPerson.create_identity (
-			firstnames = dto.firstnames,
-			lastnames = dto.lastnames,
-			gender = dto.gender,
-			dob = dto.dob
+	if idents is None:
+		gmGuiHelpers.gm_show_info (_(
+			'Cannot create new patient:\n\n'
+			' [%s %s (%s), %s]'
+			) % (dto.firstnames, dto.lastnames, dto.gender, dto.dob.strftime('%x').decode(gmI18N.get_encoding())),
+			_('Activating external patient')
 		)
-		if ident is None:
-			gmGuiHelpers.gm_show_info (
-				_(
-				'Cannot create new patient:\n\n'
-				' [%s %s (%s), %s]'
-				) % (dto.firstnames, dto.lastnames, dto.gender, dto.dob.strftime('%x').decode(gmI18N.get_encoding())),
-				_('Activating xDT patient')
-			)
-			return False
+		return False
+
+#	ident = None
+#	if len(idents) == 0:
+#		ident = gmPerson.create_identity (
+#			firstnames = dto.firstnames,
+#			lastnames = dto.lastnames,
+#			gender = dto.gender,
+#			dob = dto.dob
+#		)
+#		if ident is None:
+#			gmGuiHelpers.gm_show_info (
+#				_(
+#				'Cannot create new patient:\n\n'
+#				' [%s %s (%s), %s]'
+#				) % (dto.firstnames, dto.lastnames, dto.gender, dto.dob.strftime('%x').decode(gmI18N.get_encoding())),
+#				_('Activating xDT patient')
+#			)
+#			return False
 
 	if len(idents) == 1:
 		ident = idents[0]
@@ -539,7 +388,7 @@ def load_patient_from_external_sources(parent=None):
 		if parent is None:
 			parent = wx.GetApp().GetTopWindow()
 		dlg = cSelectPersonFromListDlg(parent=parent, id=-1)
-		dlg.set_persons(persons=persons)
+		dlg.set_persons(persons=idents)
 		result = dlg.ShowModal()
 		if result == wx.ID_CANCEL:
 			return True
@@ -553,10 +402,13 @@ def load_patient_from_external_sources(parent=None):
 			'%s %s (%s)\n'
 			'%s'
 			) % (dto.firstnames, dto.lastnames, dto.gender, dto.dob.strftime('%x').decode(gmI18N.get_encoding())),
-			_('Activating xDT patient')
+			_('Activating external patient')
 		)
 		return False
 
+	dto.import_extra_data(identity = ident)
+	dto.delete_from_source()
+
 	return True
 #============================================================
 class cPatientSelector(wx.TextCtrl):
@@ -609,7 +461,7 @@ class cPatientSelector(wx.TextCtrl):
 		self.__always_dismiss_after_search = bool ( 
 			cfg.get2 (
 				option = 'patient_search.always_dismiss_previous_patient',
-				workplace = gmPerson.gmCurrentProvider().workplace,
+				workplace = gmSurgery.gmCurrentPractice().active_workplace,
 				bias = 'user',
 				default = 0
 			)
@@ -618,7 +470,7 @@ class cPatientSelector(wx.TextCtrl):
 		self.__always_reload_after_search = bool (
 			cfg.get2 (
 				option = 'patient_search.always_reload_new_patient',
-				workplace = gmPerson.gmCurrentProvider().workplace,
+				workplace = gmSurgery.gmCurrentPractice().active_workplace,
 				bias = 'user',
 				default = 0
 			)
@@ -636,7 +488,7 @@ class cPatientSelector(wx.TextCtrl):
 		dbcfg = gmCfg.cCfgSQL()
 		dob_distance = dbcfg.get2 (
 			option = u'patient_search.dob_warn_interval',
-			workplace = gmPerson.gmCurrentProvider().workplace,
+			workplace = gmSurgery.gmCurrentPractice().active_workplace,
 			bias = u'user',
 			default = u'1 week'
 		)
@@ -644,13 +496,12 @@ class cPatientSelector(wx.TextCtrl):
 		if pat.dob_in_range(dob_distance, dob_distance):
 			now = pyDT.datetime.now(tz = gmDateTime.gmCurrentLocalTimezone)
 			enc = gmI18N.get_encoding()
-			gmGuiHelpers.gm_statustext (
-				_('%(pat)s was born on %(dow)s, %(month)s %(day)s ! (today is %(dow_now)s, %(month_now)s %(day_now)s)') % {
+			gmDispatcher.send(signal = 'statustext', msg = _(
+				'%(pat)s turns %(age)s on %(month)s %(day)s ! (today is %(month_now)s %(day_now)s)') % {
 					'pat': pat.get_description(),
-					'dow': pat['dob'].strftime('%A').decode(enc),
+					'age': pat.get_medical_age().strip('y'),
 					'month': pat['dob'].strftime('%B').decode(enc),
 					'day': pat['dob'].strftime('%d'),
-					'dow_now': now.strftime('%A').decode(enc),
 					'month_now': now.strftime('%B').decode(enc),
 					'day_now': now.strftime('%d')
 				}
@@ -661,10 +512,14 @@ class cPatientSelector(wx.TextCtrl):
 	# utility methods
 	#--------------------------------------------------------
 	def _display_name(self):
+		name = u''
+
 		if self.curr_pat.is_connected():
-			self.SetValue(self.curr_pat['description'])
-		else:
-			self.SetValue(u'')
+			name = self.curr_pat['description']
+			if self.curr_pat.locked:
+				name = _('%(name)s (locked)') % {'name': name}
+
+		self.SetValue(name)
 	#--------------------------------------------------------
 	def __remember_ident(self, ident=None):
 
@@ -690,7 +545,7 @@ class cPatientSelector(wx.TextCtrl):
 		# - process some special chars
 		wx.EVT_CHAR(self, self._on_char)
 		# - select data in input field upon tabbing in
-		wx.EVT_SET_FOCUS (self, self._on_get_focus)
+		wx.EVT_SET_FOCUS(self, self._on_get_focus)
 		# - redraw the currently active name upon losing focus
 		#   (but see the caveat in the handler)
 		# FIXME: causes core dump in one version of wxPython -SJTAN
@@ -701,7 +556,9 @@ class cPatientSelector(wx.TextCtrl):
 
 		# client internal signals
 		gmDispatcher.connect(signal=gmSignals.post_patient_selection(), receiver=self._on_post_patient_selection)
-	#----------------------------------------------
+		gmDispatcher.connect(signal = 'patient_locked', receiver = self._on_post_patient_selection)
+		gmDispatcher.connect(signal = 'patient_unlocked', receiver = self._on_post_patient_selection)
+		#----------------------------------------------
 	def _on_post_patient_selection(self, **kwargs):
 		wx.CallAfter(self._display_name)
 	#--------------------------------------------------------
@@ -790,7 +647,14 @@ class cPatientSelector(wx.TextCtrl):
 		# invoke external patient sources
 		if keycode == wx.WXK_F2:
 			evt.Skip()
-			load_patient_from_external_sources(parent=wx.GetTopLevelParent(self))
+			dbcfg = gmCfg.cCfgSQL()
+			search_immediately = bool(dbcfg.get2 (
+				option = 'patient_search.external_sources.immediately_search_if_single_source',
+				workplace = gmSurgery.gmCurrentPractice().active_workplace,
+				bias = 'user',
+				default = 0
+			))
+			load_patient_from_external_sources(parent=wx.GetTopLevelParent(self), search_immediately=search_immediately)
 			return True
 
 		# FIXME: invoke add new patient
@@ -992,8 +856,63 @@ if __name__ == "__main__":
 
 #============================================================
 # $Log: gmPatSearchWidgets.py,v $
-# Revision 1.80.2.1  2007/06/01 15:15:47  ncq
-# - every option needs a name, so use "patient_search.dob_warn_interval"
+# Revision 1.97.2.1  2008/01/08 22:39:31  ncq
+# - 'first' / 'last' -> 'firstnames' / 'lastnames'
+#
+# Revision 1.97  2007/11/12 23:05:55  ncq
+# - import extra data from DTOs
+#
+# Revision 1.96  2007/11/10 20:58:59  ncq
+# - use dto.get_candidate_identities() and dto.delete_from_source()
+#
+# Revision 1.95  2007/10/19 12:52:34  ncq
+# - implement search_immediately in load_patient_from_external_source()
+#
+# Revision 1.94  2007/10/12 14:20:09  ncq
+# - prepare "activate_immediately" in load_patient_from_external_sources()
+#
+# Revision 1.93  2007/10/12 13:33:06  ncq
+# - if only one external patient available - activate it right away
+#
+# Revision 1.92  2007/10/11 12:15:09  ncq
+# - make filling patient selector list more robust in absence of match_type field
+#
+# Revision 1.91  2007/10/07 12:32:42  ncq
+# - workplace property now on gmSurgery.gmCurrentPractice() borg
+#
+# Revision 1.90  2007/09/10 12:38:12  ncq
+# - improve wording on announcing upcoming patient birthday
+#
+# Revision 1.89  2007/08/28 14:18:13  ncq
+# - no more gm_statustext()
+#
+# Revision 1.88  2007/08/12 00:12:41  ncq
+# - no more gmSignals.py
+#
+# Revision 1.87  2007/07/17 16:00:28  ncq
+# - check existence of PracSoft import file
+#
+# Revision 1.86  2007/07/11 21:11:08  ncq
+# - display patient locked state
+# - listen on patient lock/unlock events
+#
+# Revision 1.85  2007/07/09 12:46:33  ncq
+# - move cDataMiningPnl to gmDataMiningWidgets.py
+#
+# Revision 1.84  2007/07/07 12:43:25  ncq
+# - in cDataMiningPnl use cPatientListingCtrl
+#
+# Revision 1.83  2007/06/28 12:40:48  ncq
+# - handle dto.dob being optional now
+# - support dto source gotten from xdt file
+#
+# Revision 1.82  2007/06/12 16:03:58  ncq
+# - some comments
+# - fix typo
+# - better error display on failing queries
+#
+# Revision 1.81  2007/06/10 10:12:55  ncq
+# - options need names
 #
 # Revision 1.80  2007/05/18 15:55:58  ncq
 # - auto-select first item in person/dto selector
