@@ -13,6 +13,9 @@ care of all the pre- and post-GUI runtime environment setup.
 --debug
  Pre-set the [debug mode] checkbox in the login dialog to
  increase verbosity in the log file. Useful for, well, debugging :-)
+--slave
+ Pre-set the [enable remote control] checkbox in the login
+ dialog to enable the XML-RPC remote control feature.
 --profile=<file>
  Activate profiling and write profile data to <file>.
 --text-domain=<text domain>
@@ -39,8 +42,8 @@ care of all the pre- and post-GUI runtime environment setup.
 """
 #==========================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gnumed.py,v $
-# $Id: gnumed.py,v 1.118 2007/05/21 14:49:42 ncq Exp $
-__version__ = "$Revision: 1.118 $"
+# $Id: gnumed.py,v 1.123 2007/10/21 20:21:17 ncq Exp $
+__version__ = "$Revision: 1.123 $"
 __author__  = "H. Herb <hherb@gnumed.net>, K. Hilbert <Karsten.Hilbert@gmx.net>, I. Haywood <i.haywood@ugrad.unimelb.edu.au>"
 __license__ = "GPL (details at http://www.gnu.org)"
 
@@ -77,7 +80,7 @@ warnings.filterwarnings("default", "Use\sPython.s\sTrue\sinstead", DeprecationWa
 
 
 _log = None
-_cfg = None
+#_cfg = None
 _email_logger = None
 gmLog = None
 _old_sig_hup = None
@@ -139,7 +142,7 @@ def setup_locale():
 	if gmCLI.has_arg('--lang-gettext'):
 		l = gmCLI.arg['--lang-gettext']
 
-	gmI18N.install_domain(text_domain = td, language = l)
+	gmI18N.install_domain(domain = td, language = l)
 
 	return True
 #==========================================================
@@ -150,14 +153,14 @@ def setup_paths():
 
 	gmTools.mkdir(os.path.expanduser(os.path.join('~', '.gnumed', 'scripts')))
 	gmTools.mkdir(os.path.expanduser(os.path.join('~', '.gnumed', 'spellcheck')))
+	gmTools.mkdir(os.path.expanduser(os.path.join('~', '.gnumed', 'tmp', 'docs')))
 	gmTools.mkdir(os.path.expanduser(os.path.join('~', 'gnumed', 'export', 'docs')))
 	gmTools.mkdir(os.path.expanduser(os.path.join('~', 'gnumed', 'export', 'xDT')))
 	gmTools.mkdir(os.path.expanduser(os.path.join('~', 'gnumed', 'export', 'EMR')))
-	gmTools.mkdir(os.path.expanduser(os.path.join('~', 'gnumed', 'tmp', 'docs')))
 	gmTools.mkdir(os.path.expanduser(os.path.join('~', 'gnumed', 'xDT')))
 	gmTools.mkdir(os.path.expanduser(os.path.join('~', 'gnumed', 'logs')))
 
-	paths = gmTools.cPaths()
+	paths = gmTools.gmPaths()
 #==========================================================
 def setup_date_time():
 	from Gnumed.pycommon import gmDateTime
@@ -195,17 +198,20 @@ def handle_sig_hup(signum, frame):
 		_old_sig_hup(signum, frame)
 #----------------------------------------------------------
 def handle_sig_term(signum, frame):
-	print "SIGTERM caught"
-	print signum
-	print frame
+	print 'GNUmed: SIGTERM (SIG%s) caught, shutting down ...' % signum
+	if frame is not None:
+		print '%s::%s@%s' % (frame.f_code.co_filename, frame.f_code.co_name, frame.f_lineno)
+
+	# FIXME: need to do something useful here
+
 	if _old_sig_term in [None, signal.SIG_IGN]:
 		sys.exit(signal.SIGTERM)
 	else:
 		_old_sig_term(signum, frame)
 #----------------------------------------------------------
 def setup_signal_handlers():
-	global _old_sig_hup
-	old_sig_hup = signal.signal(signal.SIGHUP, handle_sig_hup)
+#	global _old_sig_hup
+#	old_sig_hup = signal.signal(signal.SIGHUP, handle_sig_hup)
 	global _old_sig_term
 	old_sig_term = signal.signal(signal.SIGTERM, handle_sig_term)
 #==========================================================
@@ -249,13 +255,13 @@ def check_help_request():
 # main - launch the GNUmed wxPython GUI client
 #----------------------------------------------------------
 setup_console_exption_handler()
+setup_signal_handlers()
 setup_logging()
 
 from Gnumed.pycommon import gmCLI, gmI18N
 
 setup_locale()
 check_help_request()
-#setup_signal_handlers()
 
 _log.Log(gmLog.lInfo, 'Starting up as main module (%s).' % __version__)
 _log.Log(gmLog.lInfo, 'command line is: %s' % str(gmCLI.arg))
@@ -263,7 +269,7 @@ _log.Log(gmLog.lInfo, 'Python %s on %s (%s)' % (sys.version, sys.platform, os.na
 
 setup_paths()
 setup_date_time()
-setup_cfg_files()
+#setup_cfg_files()
 
 # import more of our stuff
 from Gnumed.pycommon import gmGuiBroker, gmHooks
@@ -289,6 +295,21 @@ _log.Log(gmLog.lInfo, 'Normally shutting down as main module.')
 
 #==========================================================
 # $Log: gnumed.py,v $
+# Revision 1.123  2007/10/21 20:21:17  ncq
+# - no more mandatory global config file
+#
+# Revision 1.122  2007/09/04 23:30:42  ncq
+# - explain --slave
+#
+# Revision 1.121  2007/08/07 21:42:40  ncq
+# - cPaths -> gmPaths
+#
+# Revision 1.120  2007/07/22 09:28:13  ncq
+# - tmp/ now in .gnumed/
+#
+# Revision 1.119  2007/07/13 09:12:35  ncq
+# - setup signal handler
+#
 # Revision 1.118  2007/05/21 14:49:42  ncq
 # - create gnumed/export/EMR/
 #
