@@ -12,8 +12,8 @@ The manuals should reside where the manual_path points to.
 """
 #===========================================================
 # $Source: /sources/gnumed/gnumed/gnumed/client/wxpython/gui/gmManual.py,v $
-# $Id: gmManual.py,v 1.40 2007/05/08 11:17:09 ncq Exp $
-__version__ = "$Revision: 1.40 $"
+# $Id: gmManual.py,v 1.43 2007/08/07 21:42:40 ncq Exp $
+__version__ = "$Revision: 1.43 $"
 __author__ = "H.Herb, I.Haywood, H.Berger, K.Hilbert"
 
 import os, sys, os.path
@@ -54,7 +54,7 @@ class ManualHtmlPanel(wx.Panel):
 		self.frame = frame
 
 		# get base directory for manuals from broker
-		paths = gmTools.cPaths(app_name = 'gnumed', wx = wx)
+		paths = gmTools.gmPaths(app_name = 'gnumed', wx = wx)
 		candidates = [
 			os.path.join(paths.local_base_dir, 'doc', 'user-manual'),
 			'/usr/share/doc/gnumed/user-manual/',
@@ -62,6 +62,7 @@ class ManualHtmlPanel(wx.Panel):
 		]
 		for self.docdir in candidates:
 			if os.access(self.docdir, os.R_OK):
+				_log.Log(gmLog.lInfo, 'found Manual path [%s]' % self.docdir)
 				break
 
 		self.box = wx.BoxSizer(wx.VERTICAL)
@@ -97,7 +98,7 @@ class ManualHtmlPanel(wx.Panel):
 		if os.access (name, os.F_OK):
 			self.html.LoadPage(name)
 		else:
-			_log.Log (gmLog.lErr, "cannot load local document %s" % name)
+			_log.Log(gmLog.lErr, "cannot load local document %s" % name)
 			self.html.LoadPage('http://wiki.gnumed.de/bin/view/Gnumed/GnumedManual?template=viewprint')
 	#--------------------------------------------------------
 	def OnLoadFile(self, event):
@@ -108,12 +109,10 @@ class ManualHtmlPanel(wx.Panel):
 		dlg.Destroy()
 	#--------------------------------------------------------
 	def OnBack(self, event):
-		if not self.html.HistoryBack():
-			_log.Log (gmLog.lInfo, _("ManualHtmlWindow: No more items in history!\n"))
+		self.html.HistoryBack()
 	#--------------------------------------------------------
 	def OnForward(self, event):
-		if not self.html.HistoryForward():
-			_log.Log (gmLog.lInfo, _("ManualHtmlWindow: No more items in history!\n"))
+		self.html.HistoryForward()
 	#--------------------------------------------------------
 	def OnViewSource(self, event):
 		return 1
@@ -244,6 +243,15 @@ class gmManual (gmPlugin.cNotebookPlugin):
 		wx.EVT_TOOL (tb, ID_MANUALPRINTER, widget.OnPrint) 
 #===========================================================
 # $Log: gmManual.py,v $
+# Revision 1.43  2007/08/07 21:42:40  ncq
+# - cPaths -> gmPaths
+#
+# Revision 1.42  2007/06/11 20:26:34  ncq
+# - fix spurious ":"s
+#
+# Revision 1.41  2007/06/10 10:13:33  ncq
+# - somewhat improved logging
+#
 # Revision 1.40  2007/05/08 11:17:09  ncq
 # - need to import gmTools
 #
